/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO Buy Zone
/// Version: v1.01
/// Converted to NinjaTrader by cvax
/// 
/// Changelog: 
/// v1.01 - Improved the painting method of the Buy Zones
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{

    [Description("Plots the buy zones for an instrument.")]
    [Gui.Design.DisplayName("TRO's Buy Zone")]
    public class TRO_Buy_Zone : Indicator
    {
        #region Variables
        // Wizard generated variables
		private double		spread				= .10;		//Spread between top and bottom of zones
		private double		buyzone				= .10;		//Offset from current day open
		private bool		fill				= true;		//Fill color inbetween zones
		private bool		drawopen			= true;
		
        // User defined variables (add any user defined variables below)
		private double		longtop				= 0;
		private double		longbot				= 0;
		private double		shorttop			= 0;
		private double		shortbot			= 0;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Blue, 1), PlotStyle.Hash, "Long Zone: Top"));
			Add(new Plot(new Pen(Color.Blue, 1), PlotStyle.Hash, "Long Zone: Bottom"));
			Add(new Plot(new Pen(Color.Black, 1), PlotStyle.Square, "Open"));
			Add(new Plot(new Pen(Color.Red , 1), PlotStyle.Hash, "Short Zone: Top"));
			Add(new Plot(new Pen(Color.Red , 1), PlotStyle.Hash, "Short Zone: Bottom"));
			
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= true;
			AutoScale			= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 1 || (ToDay(Time[0]) == ToDay(Time[CurrentBar])))	return;
			
			longtop = CurrentDayOHL().CurrentOpen[0]+spread+buyzone;
			longbot = CurrentDayOHL().CurrentOpen[0]+spread;
			shorttop = CurrentDayOHL().CurrentOpen[0]-spread;
			shortbot = CurrentDayOHL().CurrentOpen[0]-spread-buyzone;
			
			LongTop.Set(longtop);
			LongBottom.Set(longbot);
			if (drawopen)
				OpenValue.Set(CurrentDayOHL().CurrentOpen[0]);
			ShortTop.Set(shorttop);
			ShortBottom.Set(shortbot);
        }
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LongTop
        {
            get { return Values[0]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LongBottom
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries OpenValue
        {
            get { return Values[2]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ShortTop
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ShortBottom
        {
            get { return Values[4]; }
        }
		
		[Description("Width between the top and bottom of the Buy Zones")]
		[Category("Parameters")]
		public double BuyZoneWidth
		{
			get { return spread; }
			set { spread = Math.Max(0, value); }
		}
		
		[Description("Buy Zone's offset from the current day's Open")]
		[Category("Parameters")]
		public double BuyZoneOffset
		{
			get { return buyzone; }
			set { buyzone = Math.Max(0, value); }
		}
		
		[Description("Fill color the Buy Zones.")]
		[Category("Plots")]
		public bool ColorZone
		{
			get { return fill; }
			set { fill = value; }
		}
		
		[Description("Draw the Open line.")]
		[Category("Plots")]
		public bool DrawOpen
		{
			get { return drawopen; }
			set { drawopen = value; }
		}
		#endregion
		
		#region Miscellaneous
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			base.Plot(graphics, bounds, min, max);
			if (Bars == null || ChartControl == null)
				return;
				
			SolidBrush brushblue = new SolidBrush(Color.FromArgb(60,Color.Blue));
			SolidBrush brushred	 = new SolidBrush(Color.FromArgb(60,Color.Red));
			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			SmoothingMode oldSmoothingMode = graphics.SmoothingMode;
			GraphicsPath path = new GraphicsPath();
			GraphicsPath path2 = new GraphicsPath();
						
			if(fill)
			{
				
				//Color Long Buy Zone
				for (int seriesCount = 0; seriesCount < 2; seriesCount++)
				{
					int	lastX = -1;
					int	lastY = -1;
					DataSeries	series = (DataSeries) Values[seriesCount];
					double val = 0;
					Gui.Chart.Plot plot = Plots[seriesCount];
	
					for (int count = 0; count < ChartControl.BarsPainted; count++)
					{
						int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
						if (idx < 0 || idx >= Input.Count || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
							continue;
	
						val	= series.Get(idx);
						if (val == 0)
							continue;
	
						int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
							- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace) + 1;
						int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);
	
						if (lastX >= 0) 
						{
							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
	
						lastX	= x;
						lastY	= y;
					}
					path.Reverse();
				}
				
				//Color Short Buy Zone
				for (int seriesCount = 3; seriesCount < 5; seriesCount++)
				{
					int	lastX = -1;
					int	lastY = -1;
					DataSeries	series = (DataSeries) Values[seriesCount];
					double val = 0;
					Gui.Chart.Plot plot = Plots[seriesCount];
	
					for (int count = 0; count < ChartControl.BarsPainted; count++)
					{
						int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
						if (idx < 0 || idx >= Input.Count || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
							continue;
	
						val	= series.Get(idx);
						if (val == 0)
							continue;
	
						int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
							- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace) + 1;
						int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);
	
						if (lastX >= 0) 
						{
							path2.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
	
						lastX	= x;
						lastY	= y;
					}
					path2.Reverse();
					
				}
			}
			graphics.SmoothingMode = SmoothingMode.AntiAlias;
			graphics.SmoothingMode = oldSmoothingMode;
			graphics.FillPath(brushblue, path);
			graphics.FillPath(brushred, path2);
		}
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_Buy_Zone[] cacheTRO_Buy_Zone = null;

        private static TRO_Buy_Zone checkTRO_Buy_Zone = new TRO_Buy_Zone();

        /// <summary>
        /// Plots the buy zones for an instrument.
        /// </summary>
        /// <returns></returns>
        public TRO_Buy_Zone TRO_Buy_Zone(double buyZoneOffset, double buyZoneWidth)
        {
            return TRO_Buy_Zone(Input, buyZoneOffset, buyZoneWidth);
        }

        /// <summary>
        /// Plots the buy zones for an instrument.
        /// </summary>
        /// <returns></returns>
        public TRO_Buy_Zone TRO_Buy_Zone(Data.IDataSeries input, double buyZoneOffset, double buyZoneWidth)
        {
            checkTRO_Buy_Zone.BuyZoneOffset = buyZoneOffset;
            buyZoneOffset = checkTRO_Buy_Zone.BuyZoneOffset;
            checkTRO_Buy_Zone.BuyZoneWidth = buyZoneWidth;
            buyZoneWidth = checkTRO_Buy_Zone.BuyZoneWidth;

            if (cacheTRO_Buy_Zone != null)
                for (int idx = 0; idx < cacheTRO_Buy_Zone.Length; idx++)
                    if (Math.Abs(cacheTRO_Buy_Zone[idx].BuyZoneOffset - buyZoneOffset) <= double.Epsilon && Math.Abs(cacheTRO_Buy_Zone[idx].BuyZoneWidth - buyZoneWidth) <= double.Epsilon && cacheTRO_Buy_Zone[idx].EqualsInput(input))
                        return cacheTRO_Buy_Zone[idx];

            TRO_Buy_Zone indicator = new TRO_Buy_Zone();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.BuyZoneOffset = buyZoneOffset;
            indicator.BuyZoneWidth = buyZoneWidth;

            TRO_Buy_Zone[] tmp = new TRO_Buy_Zone[cacheTRO_Buy_Zone == null ? 1 : cacheTRO_Buy_Zone.Length + 1];
            if (cacheTRO_Buy_Zone != null)
                cacheTRO_Buy_Zone.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_Buy_Zone = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the buy zones for an instrument.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Buy_Zone TRO_Buy_Zone(double buyZoneOffset, double buyZoneWidth)
        {
            return _indicator.TRO_Buy_Zone(Input, buyZoneOffset, buyZoneWidth);
        }

        /// <summary>
        /// Plots the buy zones for an instrument.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Buy_Zone TRO_Buy_Zone(Data.IDataSeries input, double buyZoneOffset, double buyZoneWidth)
        {
            return _indicator.TRO_Buy_Zone(input, buyZoneOffset, buyZoneWidth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the buy zones for an instrument.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Buy_Zone TRO_Buy_Zone(double buyZoneOffset, double buyZoneWidth)
        {
            return _indicator.TRO_Buy_Zone(Input, buyZoneOffset, buyZoneWidth);
        }

        /// <summary>
        /// Plots the buy zones for an instrument.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Buy_Zone TRO_Buy_Zone(Data.IDataSeries input, double buyZoneOffset, double buyZoneWidth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_Buy_Zone(input, buyZoneOffset, buyZoneWidth);
        }

    }
}
#endregion
