/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO Dynamic Fibs S/R
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.02 - Fixed another bug causing S/R lines to plot improperly
/// v1.01 - Fixed a bug where S/R lines were not updating properly
/// </summary>

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Plots the dynamic support/resistance with fibonacci retracements.")]
    [Gui.Design.DisplayName("TRO's Dynamic Fibonacci S/R")]
    public class TRO_Dynamic_FibsSR : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int			period				= 5;
		private bool		showfibs			= true;
		
        // User defined variables (add any user defined variables below)
		private double		fib236				= 0;
		private double		fib382				= 0;
		private double		fib500				= 0;
		private double		fib618				= 0;
		private double		fib764				= 0;
		private double		DynamicR			= 0;
		private double		DynamicS			= 0;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Red, 3), PlotStyle.Square, "Resistance"));
			Add(new Plot(new Pen(Color.DarkCyan, 2), PlotStyle.Hash, "Fib 76.4%"));
			Add(new Plot(new Pen(Color.Gold, 2), PlotStyle.Hash, "Fib 61.8%"));
			Add(new Plot(new Pen(Color.Gray, 2), PlotStyle.Hash, "Fib 50%"));
			Add(new Plot(new Pen(Color.Gold, 2), PlotStyle.Hash, "Fib 38.2%"));
			Add(new Plot(new Pen(Color.DarkCyan, 2), PlotStyle.Hash, "Fib 23.6%"));
			Add(new Plot(new Pen(Color.Blue, 3), PlotStyle.Square, "Support"));
			
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= false;
			AutoScale			= false;
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 1 || (ToDay(Time[0]) == ToDay(Time[CurrentBar])))		return;
			
			//Reset values at start of new day
			if(Bars.DayBreak)
			{
				DynamicR			= 0;
				DynamicS			= 0;
			}
			
			//Dynamic SR
			//Use "Period-1" to match TRO's behavior in TradeStation (TradeStation considers current bar as one of the bars back while NinjaTrader checks Period back+current bar)
			int HH = Bars.HighestBar(Period-1);
			if(High[0]==High[HH] || DynamicR==0)
				DynamicR=High[HH];
			
			int LL = Bars.LowestBar(Period-1);
			if(Low[0]==Low[LL] || DynamicS==0)
				DynamicS=Low[LL];
			
			Support.Set(DynamicS);
			Resistance.Set(DynamicR);
			
			//Dynamic SR Fibs
			if(showfibs)
			{
				fib764 = (DynamicR - DynamicS) * 0.764 + DynamicS;
				fib618 = (DynamicR - DynamicS) * 0.618 + DynamicS;
				fib500 = (DynamicR + DynamicS) * 0.5;
				fib382 = (DynamicR - DynamicS) * 0.382 + DynamicS;
				fib236 = (DynamicR - DynamicS) * 0.236 + DynamicS;
				Fib764.Set(fib764);
				Fib618.Set(fib618);
				Fib500.Set(fib500);
				Fib382.Set(fib382);
				Fib236.Set(fib236);
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Resistance
        {
            get { return Values[0]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fib764
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fib618
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fib500
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fib382
        {
            get { return Values[4]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fib236
        {
            get { return Values[5]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Support
        {
            get { return Values[6]; }
        }
		
		[Description("Numbers of bars used for calculations. (Lookback period)")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
		
		[Description("Plot the fib retracements between S/R")]
		[Category("Plots")]
		public bool DrawFibs
		{
			get { return showfibs; }
			set { showfibs = value; }
		}
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_Dynamic_FibsSR[] cacheTRO_Dynamic_FibsSR = null;

        private static TRO_Dynamic_FibsSR checkTRO_Dynamic_FibsSR = new TRO_Dynamic_FibsSR();

        /// <summary>
        /// Plots the dynamic support/resistance with fibonacci retracements.
        /// </summary>
        /// <returns></returns>
        public TRO_Dynamic_FibsSR TRO_Dynamic_FibsSR(int period)
        {
            return TRO_Dynamic_FibsSR(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance with fibonacci retracements.
        /// </summary>
        /// <returns></returns>
        public TRO_Dynamic_FibsSR TRO_Dynamic_FibsSR(Data.IDataSeries input, int period)
        {
            checkTRO_Dynamic_FibsSR.Period = period;
            period = checkTRO_Dynamic_FibsSR.Period;

            if (cacheTRO_Dynamic_FibsSR != null)
                for (int idx = 0; idx < cacheTRO_Dynamic_FibsSR.Length; idx++)
                    if (cacheTRO_Dynamic_FibsSR[idx].Period == period && cacheTRO_Dynamic_FibsSR[idx].EqualsInput(input))
                        return cacheTRO_Dynamic_FibsSR[idx];

            TRO_Dynamic_FibsSR indicator = new TRO_Dynamic_FibsSR();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;

            TRO_Dynamic_FibsSR[] tmp = new TRO_Dynamic_FibsSR[cacheTRO_Dynamic_FibsSR == null ? 1 : cacheTRO_Dynamic_FibsSR.Length + 1];
            if (cacheTRO_Dynamic_FibsSR != null)
                cacheTRO_Dynamic_FibsSR.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_Dynamic_FibsSR = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the dynamic support/resistance with fibonacci retracements.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Dynamic_FibsSR TRO_Dynamic_FibsSR(int period)
        {
            return _indicator.TRO_Dynamic_FibsSR(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance with fibonacci retracements.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Dynamic_FibsSR TRO_Dynamic_FibsSR(Data.IDataSeries input, int period)
        {
            return _indicator.TRO_Dynamic_FibsSR(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the dynamic support/resistance with fibonacci retracements.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Dynamic_FibsSR TRO_Dynamic_FibsSR(int period)
        {
            return _indicator.TRO_Dynamic_FibsSR(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance with fibonacci retracements.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Dynamic_FibsSR TRO_Dynamic_FibsSR(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_Dynamic_FibsSR(input, period);
        }

    }
}
#endregion
