/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Copyright 2006 Avery T. Horton, Jr.
/// 
/// Attn: TradeStation
/// If this indicator is posted on the TradeStation Forum, I, TheRumpledOne, did NOT post it there, so I can't be blamed for this indicator having my contact info.
/// 
/// Pseudocode:
/// Expressed in words, if the high of the current bar is equal to the highest high of the last iPeriods, set the dynamic high to the current high;
/// otherwise just leave the dynamic high at whatever value it was before this bar.
/// Similarly for dynamic support.
/// 
/// Indicator: TRO Dynamic Fibs S/R2L
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.00 - Public Release
/// </summary>

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Uses Poor Man's Slot Machine to plot Trade Triggers of TRO_Dynamics_FibsSR2U.")]
    [Gui.Design.DisplayName("TRO's Dynamic Fibonacci S/R2L (PMSM)")]
    public class TRO_Dynamic_FibsSR2L : Indicator
    {
        #region Variables
        // Input Parameters
		private int			period				= 5;
		private bool		showentries			= true;
		private double		iMaxStop			= 0;
		private double		iEntryOffset		= 0;
		private double		iStopOffset			= 0;
		private double		iSROffset			= 0.01;
		private bool		paint				= false;
		
		// Entry Variables
		private double		maxshortstop		= 0;
		private double		shortstop			= 0;
		private double		shortentry			= 0;
		private double		maxlongstop			= 0;
		private double		longstop			= 0;
		private double		longentry			= 0;
		private string		lasttouch			= "";
		
        // Dynamic SR Variables
		private double		DynamicR			= 0;
		private double		DynamicS			= 0;
		private DataSeries	Support;
		private DataSeries	Resistance;
		private DataSeries	LEntry;
		private DataSeries	LStop;
		private DataSeries	SEntry;
		private DataSeries	SStop;
		
		//Plot Override
		private Color borderColor = Color.Transparent;
		private SolidBrush BuyBrush = new SolidBrush(Color.Cyan);
		private SolidBrush SellBrush = new SolidBrush(Color.Magenta);
		private SolidBrush HoldBrush = new SolidBrush(Color.Orange);
		private SolidBrush NeutralBrush = new SolidBrush(Color.Gray);
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Cyan, 2), PlotStyle.Dot, "Trade: Long Entry"));
			Add(new Plot(new Pen(Color.Orange, 2), PlotStyle.Dot, "Trade: Long Stop"));
			Add(new Plot(new Pen(Color.Magenta, 2), PlotStyle.Dot, "Trade: Short Entry"));
			Add(new Plot(new Pen(Color.Orange, 2), PlotStyle.Dot, "Trade: Short Stop"));
			
			Support = new DataSeries(this);
			Resistance = new DataSeries(this);
			LEntry = new DataSeries(this);
			LStop = new DataSeries(this);
			SEntry = new DataSeries(this);
			SStop = new DataSeries(this);
			
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= false;
			AutoScale			= false;
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 1 || (ToDay(Time[0]) == ToDay(Time[CurrentBar])))		return;
			
			//Reset values at start of new day
			if(Bars.DayBreak)
			{
				DynamicR			= 0;
				DynamicS			= 0;
			}
			
			//Dynamic SR
			//Use "Period-1" to match TRO's behavior in TradeStation (TradeStation considers current bar as one of the bars back while NinjaTrader checks Period back+current bar)
			int HH = Bars.HighestBar(Period-1);
			if(High[0]==High[HH] || DynamicR==0)
				DynamicR=High[HH];
			
			int LL = Bars.LowestBar(Period-1);
			if(Low[0]==Low[LL] || DynamicS==0)
				DynamicS=Low[LL];
			
			Support.Set(DynamicS);
			Resistance.Set(DynamicR);
			
			shortstop = DynamicR + iStopOffset;
			longstop = DynamicS - iStopOffset;
			
			if(High[0] >= (DynamicR - iSROffset))
				shortentry = Low[0] - iEntryOffset;
			
			if(Low[0] <= (DynamicS + iSROffset))
				longentry = High[0] + iEntryOffset;
			
			//Entries
			if(Low[0] == DynamicS)
				lasttouch = "S";
			else if(High[0] == DynamicR)
				lasttouch = "R";
			maxshortstop = shortstop - shortentry;
			maxlongstop = longentry - longstop;
			
			if(lasttouch == "S")
			{
				
				if((iMaxStop >= maxlongstop && iMaxStop != 0) || iMaxStop == 0)
				{
					
					LStop.Set(longstop);
					LEntry.Set(longentry);
					
					//Plot override for dot coloring: 0=ignore, 1=cyan, 2=magenta, 3=orange, 4=gray
					if(High[0]>LEntry[1] && Low[0]<=LEntry[1] && Close[0]>=Open[0])
					{
						longentryplot.Set(1);
						if(paint)	BackColor = BuyBrush.Color;
					}
					else
						longentryplot.Set(4);
					if(High[0]>=LStop[1] && Low[0]<=LStop[1])
						longstopplot.Set(3);
					else
						longstopplot.Set(4);
					
					//Tells plot override not to plot Short Trades
					shortstopplot.Set(0);
					shortentryplot.Set(0);
				}
			}
			
			if(lasttouch == "R")
			{
				if((iMaxStop >= maxlongstop && iMaxStop != 0) || iMaxStop == 0)
				{
					SStop.Set(shortstop);
					SEntry.Set(shortentry);
					
					//Plot override for dot coloring: 0=ignore, 1=cyan, 2=magenta, 3=orange, 4=gray
					if(High[0]>=SEntry[1] && Low[0]<SEntry[1] && Close[0]<=Open[0])
					{
						shortentryplot.Set(2);
						if(paint)	BackColor = SellBrush.Color;
					}
					else
						shortentryplot.Set(4);
					if(High[0]>=SStop[1] && Low[0]<=SStop[1])
						shortstopplot.Set(3);
					else
						shortstopplot.Set(4);
					
					//Tells plot override not to plot Long Trades
					longstopplot.Set(0);
					longentryplot.Set(0);
				}
			}
        }

		#region Plot Override
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. 
			//base.Plot(graphics, bounds, min, max);

			if (base.Bars == null) return;

			Exception caughtException;
			
        	int index 	= -1;
			byte bRed 	= (byte)~(base.ChartControl.BackColor.R);
            byte bGreen = (byte)~(base.ChartControl.BackColor.G);
			byte bBlue 	= (byte)~(base.ChartControl.BackColor.B);
			
			
			borderColor = Color.FromArgb(bRed,bGreen,bBlue);
			
			
			int bars = base.ChartControl.BarsPainted;
            int barPaintWidth = base.ChartControl.ChartStyle.GetBarPaintWidth(base.ChartControl.BarWidth);

            while (bars >= 0)
            {
				index = ((base.ChartControl.LastBarPainted - base.ChartControl.BarsPainted) +1) + bars;
                if (base.ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired))
                {
		  			try
					{
						int plot = 4;
						while(plot>0)
						{
							double alertVal = 0;
							if(plot==4)
								alertVal = longentryplot.Get(index);
							else if(plot==3)
								alertVal = longstopplot.Get(index);
							else if(plot==2)
								alertVal = shortentryplot.Get(index);
							else if(plot==1)
								alertVal = shortstopplot.Get(index);
							if (alertVal != 0) 
							{
								int x = (((base.ChartControl.CanvasRight - base.ChartControl.BarMarginRight) - (barPaintWidth / 2)) - ((base.ChartControl.BarsPainted - 1) * base.ChartControl.BarSpace)) + (bars * base.ChartControl.BarSpace);
								int y = 0;
								if(plot==4)
									y = (bounds.Top+7);
								else if(plot==3)
									y = (bounds.Top+7)+(bounds.Bottom-7-(bounds.Top+7))/3;
								else if(plot==2)
									y = (bounds.Top+7)+2*((bounds.Bottom-7-(bounds.Top+7))/3);
								else if(plot==1)
									y = (bounds.Bottom-7);
								Color dotColor = Color.Transparent;
								
								//Coloring: 0=ignore, 1=cyan, 2=magenta, 3=orange, 4=gray
								if (1==alertVal) dotColor = BuyBrush.Color;
								else if (2==alertVal) dotColor = SellBrush.Color;
								else if (3==alertVal) dotColor = HoldBrush.Color;
								else dotColor = NeutralBrush.Color;
								
								if (dotColor!=Color.Transparent)
								{								     
									SmoothingMode smoothingMode = graphics.SmoothingMode;
								graphics.SmoothingMode = SmoothingMode.AntiAlias;
									
									Pen pen = new Pen(borderColor, 7);
									pen.DashStyle = DashStyle.Dot;
									pen.DashCap = DashCap.Round;
													
									graphics.DrawPie(pen,x-1,y-1,2,2,0,360);
													
									pen = new Pen(dotColor, 5);
									pen.DashStyle = DashStyle.Dot;
									pen.DashCap = DashCap.Round;
									
									graphics.DrawRectangle(pen,x-2,y-2,2,2);

									graphics.SmoothingMode = smoothingMode; 
								}
							}
							plot--;
						}						
					}
					catch (Exception exception) {caughtException=exception;}
				}
				bars--;
			}
		}
		#endregion
		
        #region Properties
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries longentryplot
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries longstopplot
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries shortentryplot
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries shortstopplot
        {
            get { return Values[3]; }
        }
		
		[Description("Numbers of bars used for calculations. (Lookback period)")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
		
		[Description("Nudge factor for the Entry Prices")]
		[Category("Entries")]
		[Gui.Design.DisplayName("Entry Offset")]
		public double EntryOffset
		{
			get { return iEntryOffset; }
			set { iEntryOffset = Math.Max(0,value); }
		}
		
		[Description("Nudge factor for the Stop Prices")]
		[Category("Entries")]
		[Gui.Design.DisplayName("Stop Offset")]
		public double StopOffset
		{
			get { return iStopOffset; }
			set { iStopOffset = Math.Max(0,value); }
		}
		
		[Description("Nudge factor for deciding if touched S/R")]
		[Category("Entries")]
		[Gui.Design.DisplayName("Touch SR Offset")]
		public double SROffset
		{
			get { return iSROffset; }
			set { iSROffset = Math.Max(0,value); }
		}
		
		[Description("Value for maximum stop (0=default stop)")]
		[Category("Entries")]
		[Gui.Design.DisplayName("Max Stop")]
		public double MaxStop
		{
			get { return iMaxStop; }
			set { iMaxStop = Math.Max(0,value); }
		}
		
		[Description("On/Off for painting the background according to the entries")]
		[Category("Plots")]
		[Gui.Design.DisplayName("Paint Background")]
		public bool Paint
		{
			get { return paint; }
			set { paint = value; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_Dynamic_FibsSR2L[] cacheTRO_Dynamic_FibsSR2L = null;

        private static TRO_Dynamic_FibsSR2L checkTRO_Dynamic_FibsSR2L = new TRO_Dynamic_FibsSR2L();

        /// <summary>
        /// Uses Poor Man's Slot Machine to plot Trade Triggers of TRO_Dynamics_FibsSR2U.
        /// </summary>
        /// <returns></returns>
        public TRO_Dynamic_FibsSR2L TRO_Dynamic_FibsSR2L(int period)
        {
            return TRO_Dynamic_FibsSR2L(Input, period);
        }

        /// <summary>
        /// Uses Poor Man's Slot Machine to plot Trade Triggers of TRO_Dynamics_FibsSR2U.
        /// </summary>
        /// <returns></returns>
        public TRO_Dynamic_FibsSR2L TRO_Dynamic_FibsSR2L(Data.IDataSeries input, int period)
        {
            checkTRO_Dynamic_FibsSR2L.Period = period;
            period = checkTRO_Dynamic_FibsSR2L.Period;

            if (cacheTRO_Dynamic_FibsSR2L != null)
                for (int idx = 0; idx < cacheTRO_Dynamic_FibsSR2L.Length; idx++)
                    if (cacheTRO_Dynamic_FibsSR2L[idx].Period == period && cacheTRO_Dynamic_FibsSR2L[idx].EqualsInput(input))
                        return cacheTRO_Dynamic_FibsSR2L[idx];

            TRO_Dynamic_FibsSR2L indicator = new TRO_Dynamic_FibsSR2L();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;

            TRO_Dynamic_FibsSR2L[] tmp = new TRO_Dynamic_FibsSR2L[cacheTRO_Dynamic_FibsSR2L == null ? 1 : cacheTRO_Dynamic_FibsSR2L.Length + 1];
            if (cacheTRO_Dynamic_FibsSR2L != null)
                cacheTRO_Dynamic_FibsSR2L.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_Dynamic_FibsSR2L = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Uses Poor Man's Slot Machine to plot Trade Triggers of TRO_Dynamics_FibsSR2U.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Dynamic_FibsSR2L TRO_Dynamic_FibsSR2L(int period)
        {
            return _indicator.TRO_Dynamic_FibsSR2L(Input, period);
        }

        /// <summary>
        /// Uses Poor Man's Slot Machine to plot Trade Triggers of TRO_Dynamics_FibsSR2U.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Dynamic_FibsSR2L TRO_Dynamic_FibsSR2L(Data.IDataSeries input, int period)
        {
            return _indicator.TRO_Dynamic_FibsSR2L(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Uses Poor Man's Slot Machine to plot Trade Triggers of TRO_Dynamics_FibsSR2U.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Dynamic_FibsSR2L TRO_Dynamic_FibsSR2L(int period)
        {
            return _indicator.TRO_Dynamic_FibsSR2L(Input, period);
        }

        /// <summary>
        /// Uses Poor Man's Slot Machine to plot Trade Triggers of TRO_Dynamics_FibsSR2U.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Dynamic_FibsSR2L TRO_Dynamic_FibsSR2L(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_Dynamic_FibsSR2L(input, period);
        }

    }
}
#endregion
