/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO Dynamic S/R 3
/// Converted to NinjaTrader by cvax
/// 
/// Usage:
/// Enter Long when price drops to make new support then retraces back pass cyan line.
/// Enter Short when price climbs to make new resistance then retraces back down pass magenta line.
/// 
/// Changelog:
/// v1.10 - Added plots for Long/Short entry triggers
///		  - Fixed another bug causing S/R lines to plot improperly
/// v1.01 - Fixed a bug where S/R lines were not updating properly
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Plots the dynamic support/resistance and entry triggers.")]
    [Gui.Design.DisplayName("TRO's Dynamic Support/Resistance 3")]
    public class TRO_Dynamic_SR3 : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int			period				= 5;
		private bool		showmidpoint		= true;
		private bool		showentries			= true;
		
        // User defined variables (add any user defined variables below)
		private double		DynamicM			= 0;
		private double		DynamicR			= 0;
		private double		DynamicS			= 0;
		
		private double		LgEntry				= 0;
		private double		ShEntry				= 0;
		
		private int			barcounter			= 0;		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Cyan,2), PlotStyle.Hash, "Dyn Long"));
			Add(new Plot(new Pen(Color.Magenta,2), PlotStyle.Hash, "Dyn Short"));
			Add(new Plot(new Pen(Color.Red,2), PlotStyle.Square, "Resistance"));
			Add(new Plot(new Pen(Color.Olive,1), PlotStyle.Dot, "Middle"));
			Add(new Plot(new Pen(Color.Blue,2), PlotStyle.Square, "Support"));
			
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= false;
			AutoScale			= false;
			BarsRequired		= 1;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 1 || (ToDay(Time[0]) == ToDay(Time[CurrentBar])))	return;
			
			//Reset values at start of new day
			if(Bars.DayBreak)
			{
				DynamicM			= 0;
				DynamicR			= 0;
				DynamicS			= 0;
				LgEntry				= 0;
				ShEntry				= 0;
			}

			//Dynamic SR
			//Use "Period-1" to match TRO's behavior in TradeStation (TradeStation considers current bar as one of the bars back while NinjaTrader checks Period back+current bar)
			int HH = Bars.HighestBar(Period-1);
			if(High[0]==High[HH] || DynamicR==0)
				DynamicR=High[HH];
			
			int LL = Bars.LowestBar(Period-1);
			if(Low[0]==Low[LL] || DynamicS==0)
				DynamicS=Low[LL];
			
			Support.Set(DynamicS);
			Resistance.Set(DynamicR);
			
			//Dynamic SR Midpoint
			if(showmidpoint)
			{
				DynamicM = (DynamicR + DynamicS) * 0.5;	
				Middle.Set(DynamicM);
			}
			
			//Dynamic Entries
			if(showentries)
			{
				if (High[0] == DynamicR || DynamicR != Resistance[1])
					ShEntry = Low[0];
				else if (DynamicR == Resistance[1])
					ShEntry = DynShort[1];
				
				if (Low[0] == DynamicS || DynamicS != Support[1])
					LgEntry = High[0];
				else if (DynamicS == Support[1])
					LgEntry = DynLong[1];
				DynLong.Set(LgEntry);
				DynShort.Set(ShEntry);
			}
        }

        #region Properties
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DynLong
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DynShort
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Resistance
        {
            get { return Values[2]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Middle
        {
            get { return Values[3]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Support
        {
            get { return Values[4]; }
        }
		
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
		
		[Description("Plot the midpoint between S/R")]
		[Category("Plots")]
		public bool DrawMiddle
		{
			get { return showmidpoint; }
			set { showmidpoint = value; }
		}
		
		[Description("Plot the entry signals between S/R")]
		[Category("Plots")]
		public bool DrawEntries
		{
			get { return showentries; }
			set { showentries = value; }
		}		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_Dynamic_SR3[] cacheTRO_Dynamic_SR3 = null;

        private static TRO_Dynamic_SR3 checkTRO_Dynamic_SR3 = new TRO_Dynamic_SR3();

        /// <summary>
        /// Plots the dynamic support/resistance and entry triggers.
        /// </summary>
        /// <returns></returns>
        public TRO_Dynamic_SR3 TRO_Dynamic_SR3(int period)
        {
            return TRO_Dynamic_SR3(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance and entry triggers.
        /// </summary>
        /// <returns></returns>
        public TRO_Dynamic_SR3 TRO_Dynamic_SR3(Data.IDataSeries input, int period)
        {
            checkTRO_Dynamic_SR3.Period = period;
            period = checkTRO_Dynamic_SR3.Period;

            if (cacheTRO_Dynamic_SR3 != null)
                for (int idx = 0; idx < cacheTRO_Dynamic_SR3.Length; idx++)
                    if (cacheTRO_Dynamic_SR3[idx].Period == period && cacheTRO_Dynamic_SR3[idx].EqualsInput(input))
                        return cacheTRO_Dynamic_SR3[idx];

            TRO_Dynamic_SR3 indicator = new TRO_Dynamic_SR3();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;

            TRO_Dynamic_SR3[] tmp = new TRO_Dynamic_SR3[cacheTRO_Dynamic_SR3 == null ? 1 : cacheTRO_Dynamic_SR3.Length + 1];
            if (cacheTRO_Dynamic_SR3 != null)
                cacheTRO_Dynamic_SR3.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_Dynamic_SR3 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the dynamic support/resistance and entry triggers.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Dynamic_SR3 TRO_Dynamic_SR3(int period)
        {
            return _indicator.TRO_Dynamic_SR3(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance and entry triggers.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Dynamic_SR3 TRO_Dynamic_SR3(Data.IDataSeries input, int period)
        {
            return _indicator.TRO_Dynamic_SR3(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the dynamic support/resistance and entry triggers.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_Dynamic_SR3 TRO_Dynamic_SR3(int period)
        {
            return _indicator.TRO_Dynamic_SR3(Input, period);
        }

        /// <summary>
        /// Plots the dynamic support/resistance and entry triggers.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_Dynamic_SR3 TRO_Dynamic_SR3(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_Dynamic_SR3(input, period);
        }

    }
}
#endregion
