/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO PMSM High Low
/// Version: 1.0
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.01 - Revamped plotting method to use Gumphrie's dots
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Plots higher highs and lower lows in a linear display.")]
    [Gui.Design.DisplayName("TRO's Poor Man's Slot Machine High/Low")]
    public class TRO_PMSM_HL : Indicator
    {
        #region Variables
		private bool	paint				= true;
		private Color borderColor = Color.Transparent;
		private SolidBrush BuyBrush = new SolidBrush(Color.LimeGreen);
		private SolidBrush SellBrush = new SolidBrush(Color.Red);
		private SolidBrush HoldBrush = new SolidBrush(Color.Gold);
		
        #endregion
        protected override void Initialize()
        {
			Add(new Plot(Color.Green, PlotStyle.Dot, "High"));
			Add(new Plot(Color.Red, PlotStyle.Dot, "Low"));
			Add(new Line(new Pen(Color.Black, 1), 0, "Zero Line"));
			
            CalculateOnBarClose	= false;
			DisplayInDataBox 	= false;
            Overlay				= false;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= false;
			AutoScale			= false;
			BarsRequired		= 1;
			
        }

        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 1)	return;
			
			//Highs
			if (High[0] > High[1])
				HigherHighs.Set(1);
			else if (High[0] < High[1])
				HigherHighs.Set(-1);
			else
				HigherHighs.Set(0);
			
			//Lows
			if (Low[0] > Low[1])
				LowerLows.Set(1);
			else if (Low[0] < Low[1])
				LowerLows.Set(-1);
			else
				LowerLows.Set(0);
			
			//Paint Matching Conditions
			if (paint)
			{
				if (HigherHighs[0] == LowerLows[0] && LowerLows[0] == 1)
					BackColor = Color.PaleGreen;
				
				if (HigherHighs[0] == LowerLows[0] && LowerLows[0] == -1)
					BackColor = Color.Pink;
				
				if (HigherHighs[0] == LowerLows[0] && LowerLows[0] == 0)
					BackColor = Color.Khaki;
			}
        }

		#region Plot Override
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. 
			//base.Plot(graphics, bounds, min, max);

			if (base.Bars == null) return;

			Exception caughtException;
			
        	int index 	= -1;
			byte bRed 	= (byte)~(base.ChartControl.BackColor.R);
            byte bGreen = (byte)~(base.ChartControl.BackColor.G);
			byte bBlue 	= (byte)~(base.ChartControl.BackColor.B);
			
			
			borderColor = Color.FromArgb(bRed,bGreen,bBlue);
			
			
			int bars = base.ChartControl.BarsPainted;
            int barPaintWidth = base.ChartControl.ChartStyle.GetBarPaintWidth(base.ChartControl.BarWidth);

            while (bars >= 0)
            {
				index = ((base.ChartControl.LastBarPainted - base.ChartControl.BarsPainted) +1) + bars;
                if (base.ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired))
                {
		  			try
					{
						int plot = 2;
						while(plot>0)
						{
							double alertVal = 0;
							if(plot==2)
								alertVal = HigherHighs.Get(index);
							else if(plot==1)
								alertVal = LowerLows.Get(index);
							if (!double.IsNaN(alertVal)) 
							{
								int x = (((base.ChartControl.CanvasRight - base.ChartControl.BarMarginRight) - (barPaintWidth / 2)) - ((base.ChartControl.BarsPainted - 1) * base.ChartControl.BarSpace)) + (bars * base.ChartControl.BarSpace);
								int y = 0;
								if(plot==2)
									y = (bounds.Top+7);
								else if(plot==1)
									y = (bounds.Bottom-7);
								Color dotColor = Color.Transparent;
								
								if (1==alertVal) dotColor = BuyBrush.Color;
								else if (-1==alertVal) dotColor = SellBrush.Color;
								else dotColor = HoldBrush.Color;
								
								if (dotColor!=Color.Transparent)
								{								     
									SmoothingMode smoothingMode = graphics.SmoothingMode;
								graphics.SmoothingMode = SmoothingMode.AntiAlias;
									
									Pen pen = new Pen(borderColor, 7);
									pen.DashStyle = DashStyle.Dot;
									pen.DashCap = DashCap.Round;
													
									graphics.DrawPie(pen,x-1,y-1,2,2,0,360);
													
									pen = new Pen(dotColor, 5);
									pen.DashStyle = DashStyle.Dot;
									pen.DashCap = DashCap.Round;
									
									graphics.DrawRectangle(pen,x-2,y-2,2,2);
									//graphics.DrawPie(pen,x,y-1,2,2,0,360);
									
									
									graphics.SmoothingMode = smoothingMode; 
								}
							}
							plot--;
						}						
					}
					catch (Exception exception) {caughtException=exception;}
				}
				bars--;
			}
		}
		#endregion

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries HigherHighs
        {
            get { return Values[0]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LowerLows
        {
            get { return Values[1]; }
        }
		
		[Description("Paint trending conditions On/Off")]
		[Category("Plots")]
		public bool ShowPaint
		{
			get { return paint; }
			set { paint = value; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_PMSM_HL[] cacheTRO_PMSM_HL = null;

        private static TRO_PMSM_HL checkTRO_PMSM_HL = new TRO_PMSM_HL();

        /// <summary>
        /// Plots higher highs and lower lows in a linear display.
        /// </summary>
        /// <returns></returns>
        public TRO_PMSM_HL TRO_PMSM_HL()
        {
            return TRO_PMSM_HL(Input);
        }

        /// <summary>
        /// Plots higher highs and lower lows in a linear display.
        /// </summary>
        /// <returns></returns>
        public TRO_PMSM_HL TRO_PMSM_HL(Data.IDataSeries input)
        {

            if (cacheTRO_PMSM_HL != null)
                for (int idx = 0; idx < cacheTRO_PMSM_HL.Length; idx++)
                    if (cacheTRO_PMSM_HL[idx].EqualsInput(input))
                        return cacheTRO_PMSM_HL[idx];

            TRO_PMSM_HL indicator = new TRO_PMSM_HL();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            TRO_PMSM_HL[] tmp = new TRO_PMSM_HL[cacheTRO_PMSM_HL == null ? 1 : cacheTRO_PMSM_HL.Length + 1];
            if (cacheTRO_PMSM_HL != null)
                cacheTRO_PMSM_HL.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_PMSM_HL = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots higher highs and lower lows in a linear display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_PMSM_HL TRO_PMSM_HL()
        {
            return _indicator.TRO_PMSM_HL(Input);
        }

        /// <summary>
        /// Plots higher highs and lower lows in a linear display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_PMSM_HL TRO_PMSM_HL(Data.IDataSeries input)
        {
            return _indicator.TRO_PMSM_HL(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots higher highs and lower lows in a linear display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_PMSM_HL TRO_PMSM_HL()
        {
            return _indicator.TRO_PMSM_HL(Input);
        }

        /// <summary>
        /// Plots higher highs and lower lows in a linear display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_PMSM_HL TRO_PMSM_HL(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_PMSM_HL(input);
        }

    }
}
#endregion
