/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO PMSM Trend
/// Version: 1.11
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.11 - Revamped plotting method to use Gumphrie's dots
/// v1.10 - Added ability to choose moving average type
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Plots the trend of moving averages in a linear display.")]
    [Gui.Design.DisplayName("TRO's Poor Man's Slot Machine Trend")]
    public class TRO_PMSM_Trend : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int		ma1					= 5;		//Default period for first SMA
		private int		ma2					= 10;		//Default period for second SMA
		private int		ma3					= 20;		//Default period for third SMA
		private int		ma4					= 40;		//Default period for fourth SMA
		private bool	paint				= true;		//Default setting for paint conditions
		private TROAverageType	matype		= TROAverageType.EMA;
		
        // User defined variables (add any user defined variables below)
		private int		ma1value			= 0;
		private int		ma2value			= 0;
		private int		ma3value			= 0;
		private int		ma4value			= 0;
		
		//Plot Override
		private Color borderColor = Color.Transparent;
		private SolidBrush BuyBrush = new SolidBrush(Color.LimeGreen);
		private SolidBrush SellBrush = new SolidBrush(Color.Red);
		private SolidBrush HoldBrush = new SolidBrush(Color.Gold);
        #endregion
		
        protected override void Initialize()
        {
			Add(new Plot(Color.Black, PlotStyle.Dot, "1st MA"));
			Add(new Plot(Color.Black, PlotStyle.Dot, "2nd MA"));
			Add(new Plot(Color.Black, PlotStyle.Dot, "3rd MA"));
			Add(new Plot(Color.Black, PlotStyle.Dot, "4th MA"));

            CalculateOnBarClose	= false;
			DisplayInDataBox	= false;
            Overlay				= false;
            PriceTypeSupported	= true;
			PaintPriceMarkers	= false;
			BarsRequired		= 20;
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 20)	return;
			
			switch (matype)
			{
				case TROAverageType.EMA:
				{
					#region EMA
					//1st EMA
					if (EMA(ma1)[0] > EMA(ma1)[1])
						MA1st.Set(1);
					else if (EMA(ma1)[0] < EMA(ma1)[1])
						MA1st.Set(-1);
					else
						MA1st.Set(0);
					
					//2nd EMA
					if (EMA(ma2)[0] > EMA(ma2)[1])
						MA2nd.Set(1);
					else if (EMA(ma2)[0] < EMA(ma2)[1])
						MA2nd.Set(-1);
					else
						MA2nd.Set(0);
					
					//3rd EMA
					if (EMA(ma3)[0] > EMA(ma3)[1])
						MA3rd.Set(1);
					else if (EMA(ma3)[0] < EMA(ma3)[1])
						MA3rd.Set(-1);
					else
						MA3rd.Set(0);
					
					//4th EMA
					if (EMA(ma4)[0] > EMA(ma4)[1])
						MA4th.Set(1);
					else if (EMA(ma4)[0] < EMA(ma4)[1])
						MA4th.Set(-1);
					else
						MA4th.Set(0);
					#endregion
					break;
				}
				case TROAverageType.HMA:
				{
					#region HMA
					//1st HMA
					if (HMA(ma1)[0] > HMA(ma1)[1])
						MA1st.Set(1);
					else if (HMA(ma1)[0] < HMA(ma1)[1])
						MA1st.Set(-1);
					else
						MA1st.Set(0);
					
					//2nd HMA
					if (HMA(ma2)[0] > HMA(ma2)[1])
						MA2nd.Set(1);
					else if (HMA(ma2)[0] < HMA(ma2)[1])
						MA2nd.Set(-1);
					else
						MA2nd.Set(0);
					
					//3rd HMA
					if (HMA(ma3)[0] > HMA(ma3)[1])
						MA3rd.Set(1);
					else if (HMA(ma3)[0] < HMA(ma3)[1])
						MA3rd.Set(-1);
					else
						MA3rd.Set(0);
					
					//4th HMA
					if (HMA(ma4)[0] > HMA(ma4)[1])
						MA4th.Set(1);
					else if (HMA(ma4)[0] < HMA(ma4)[1])
						MA4th.Set(-1);
					else
						MA4th.Set(0);
					#endregion
					break;
				}
				case TROAverageType.SMA:
				{
					#region SMA
					//1st SMA
					if (SMA(ma1)[0] > SMA(ma1)[1])
						MA1st.Set(1);
					else if (SMA(ma1)[0] < SMA(ma1)[1])
						MA1st.Set(-1);
					else
						MA1st.Set(0);
					
					//2nd SMA
					if (SMA(ma2)[0] > SMA(ma2)[1])
						MA2nd.Set(1);
					else if (SMA(ma2)[0] < SMA(ma2)[1])
						MA2nd.Set(-1);
					else
						MA2nd.Set(0);
					
					//3rd SMA
					if (SMA(ma3)[0] > SMA(ma3)[1])
						MA3rd.Set(1);
					else if (SMA(ma3)[0] < SMA(ma3)[1])
						MA3rd.Set(-1);
					else
						MA3rd.Set(0);
					
					//4th SMA
					if (SMA(ma4)[0] > SMA(ma4)[1])
						MA4th.Set(1);
					else if (SMA(ma4)[0] < SMA(ma4)[1])
						MA4th.Set(-1);
					else
						MA4th.Set(0);
					#endregion
					break;
				}
				case TROAverageType.WMA:
				{
					#region WMA
					//1st WMA
					if (WMA(ma1)[0] > WMA(ma1)[1])
					{
						MA1st.Set(1);
						ma1value = 1;
					}
					else if (WMA(ma1)[0] < WMA(ma1)[1])
					{
						MA1st.Set(-1);
						ma1value = -1;
					}
					else
					{
						MA1st.Set(0);
						ma1value = 0;
					}
					
					//2nd WMA
					if (WMA(ma2)[0] > WMA(ma2)[1])
					{
						MA2nd.Set(1);
						ma2value = 1;
					}
					else if (WMA(ma2)[0] < WMA(ma2)[1])
					{
						MA2nd.Set(-1);
						ma2value = -1;
					}
					else
					{
						MA2nd.Set(0);
						ma2value = 0;
					}
					
					//3rd WMA
					if (WMA(ma3)[0] > WMA(ma3)[1])
					{
						MA3rd.Set(1);
						ma3value = 1;
					}
					else if (WMA(ma3)[0] < WMA(ma3)[1])
					{
						MA3rd.Set(-1);
						ma3value = -1;
					}
					else
					{
						MA3rd.Set(0);
						ma3value = 0;
					}
					
					//4th WMA
					if (WMA(ma4)[0] > WMA(ma4)[1])
					{
						MA4th.Set(1);
						ma4value = 1;
					}
					else if (WMA(ma4)[0] < WMA(ma4)[1])
					{
						MA4th.Set(-1);
						ma4value = -1;
					}
					else
					{
						MA4th.Set(0);
						ma4value = 0;
					}
					#endregion
					break;
				}
			}
			
			//Paint Matching Conditions
			if (paint)
			{
				if (MA1st[0] == MA2nd[0] && MA2nd[0] == MA3rd[0] && MA3rd[0] == MA4th[0] && MA4th[0] == 1)
					BackColor = Color.PaleGreen;
				
				if (MA1st[0] == MA2nd[0] && MA2nd[0] == MA3rd[0] && MA3rd[0] == MA4th[0] && MA4th[0] == -1)
					BackColor = Color.Pink;
				
				if (MA1st[0] == MA2nd[0] && MA2nd[0] == MA3rd[0] && MA3rd[0] == MA4th[0] && MA4th[0] == 0)
					BackColor = Color.Khaki;
			}
        }

		#region Plot Override
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. 
			//base.Plot(graphics, bounds, min, max);

			if (base.Bars == null) return;

			Exception caughtException;
			
        	int index 	= -1;
			byte bRed 	= (byte)~(base.ChartControl.BackColor.R);
            byte bGreen = (byte)~(base.ChartControl.BackColor.G);
			byte bBlue 	= (byte)~(base.ChartControl.BackColor.B);
			
			
			borderColor = Color.FromArgb(bRed,bGreen,bBlue);
			
			
			int bars = base.ChartControl.BarsPainted;
            int barPaintWidth = base.ChartControl.ChartStyle.GetBarPaintWidth(base.ChartControl.BarWidth);

            while (bars >= 0)
            {
				index = ((base.ChartControl.LastBarPainted - base.ChartControl.BarsPainted) +1) + bars;
                if (base.ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired))
                {
		  			try
					{
						int plot = 4;
						while(plot>0)
						{
							double alertVal = 0;
							if(plot==4)
								alertVal = MA1st.Get(index);
							else if(plot==3)
								alertVal = MA2nd.Get(index);
							else if(plot==2)
								alertVal = MA3rd.Get(index);
							else if(plot==1)
								alertVal = MA4th.Get(index);
							if (!double.IsNaN(alertVal)) 
							{
								int x = (((base.ChartControl.CanvasRight - base.ChartControl.BarMarginRight) - (barPaintWidth / 2)) - ((base.ChartControl.BarsPainted - 1) * base.ChartControl.BarSpace)) + (bars * base.ChartControl.BarSpace);
								int y = 0;
								if(plot==4)
									y = (bounds.Top+7);
								else if(plot==3)
									y = (bounds.Top+7)+(bounds.Bottom-7-(bounds.Top+7))/3;
								else if(plot==2)
									y = (bounds.Top+7)+2*((bounds.Bottom-7-(bounds.Top+7))/3);
								else if(plot==1)
									y = (bounds.Bottom-7);
								Color dotColor = Color.Transparent;
								
								if (1==alertVal) dotColor = BuyBrush.Color;
								else if (-1==alertVal) dotColor = SellBrush.Color;
								else dotColor = HoldBrush.Color;
								
								if (dotColor!=Color.Transparent)
								{								     
									SmoothingMode smoothingMode = graphics.SmoothingMode;
								graphics.SmoothingMode = SmoothingMode.AntiAlias;
									
									Pen pen = new Pen(borderColor, 7);
									pen.DashStyle = DashStyle.Dot;
									pen.DashCap = DashCap.Round;
													
									graphics.DrawPie(pen,x-1,y-1,2,2,0,360);
													
									pen = new Pen(dotColor, 5);
									pen.DashStyle = DashStyle.Dot;
									pen.DashCap = DashCap.Round;
									
									graphics.DrawRectangle(pen,x-2,y-2,2,2);

									graphics.SmoothingMode = smoothingMode; 
								}
							}
							plot--;
						}						
					}
					catch (Exception exception) {caughtException=exception;}
				}
				bars--;
			}
		}
		#endregion

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MA1st
        {
            get { return Values[0]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MA2nd
        {
            get { return Values[1]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MA3rd
        {
            get { return Values[2]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MA4th
        {
            get { return Values[3]; }
        }
		
		[Description("Period for first SMA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA1
		{
			get { return ma1; }
			set { ma1 = Math.Max(1, value); }
		}

		[Description("Period for second SMA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA2
		{
			get { return ma2; }
			set { ma2 = Math.Max(1, value); }
		}
		
		[Description("Period for third SMA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA3
		{
			get { return ma3; }
			set { ma3 = Math.Max(1, value); }
		}
		
		[Description("Period for fourth SMA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA4
		{
			get { return ma4; }
			set { ma4 = Math.Max(1, value); }
		}

		[Description("Choose a Moving Average type.")]
		[Category("Parameters")]
		public TROAverageType MAType
		{
			get { return matype; }
			set { matype = value; }
		}
		
		[Description("Paint trending conditions On/Off")]
		[Category("General")]
		public bool ShowPaint
		{
			get { return paint; }
			set { paint = value; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_PMSM_Trend[] cacheTRO_PMSM_Trend = null;

        private static TRO_PMSM_Trend checkTRO_PMSM_Trend = new TRO_PMSM_Trend();

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public TRO_PMSM_Trend TRO_PMSM_Trend(int mA1, int mA2, int mA3, int mA4)
        {
            return TRO_PMSM_Trend(Input, mA1, mA2, mA3, mA4);
        }

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public TRO_PMSM_Trend TRO_PMSM_Trend(Data.IDataSeries input, int mA1, int mA2, int mA3, int mA4)
        {
            checkTRO_PMSM_Trend.MA1 = mA1;
            mA1 = checkTRO_PMSM_Trend.MA1;
            checkTRO_PMSM_Trend.MA2 = mA2;
            mA2 = checkTRO_PMSM_Trend.MA2;
            checkTRO_PMSM_Trend.MA3 = mA3;
            mA3 = checkTRO_PMSM_Trend.MA3;
            checkTRO_PMSM_Trend.MA4 = mA4;
            mA4 = checkTRO_PMSM_Trend.MA4;

            if (cacheTRO_PMSM_Trend != null)
                for (int idx = 0; idx < cacheTRO_PMSM_Trend.Length; idx++)
                    if (cacheTRO_PMSM_Trend[idx].MA1 == mA1 && cacheTRO_PMSM_Trend[idx].MA2 == mA2 && cacheTRO_PMSM_Trend[idx].MA3 == mA3 && cacheTRO_PMSM_Trend[idx].MA4 == mA4 && cacheTRO_PMSM_Trend[idx].EqualsInput(input))
                        return cacheTRO_PMSM_Trend[idx];

            TRO_PMSM_Trend indicator = new TRO_PMSM_Trend();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MA1 = mA1;
            indicator.MA2 = mA2;
            indicator.MA3 = mA3;
            indicator.MA4 = mA4;

            TRO_PMSM_Trend[] tmp = new TRO_PMSM_Trend[cacheTRO_PMSM_Trend == null ? 1 : cacheTRO_PMSM_Trend.Length + 1];
            if (cacheTRO_PMSM_Trend != null)
                cacheTRO_PMSM_Trend.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_PMSM_Trend = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_PMSM_Trend TRO_PMSM_Trend(int mA1, int mA2, int mA3, int mA4)
        {
            return _indicator.TRO_PMSM_Trend(Input, mA1, mA2, mA3, mA4);
        }

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_PMSM_Trend TRO_PMSM_Trend(Data.IDataSeries input, int mA1, int mA2, int mA3, int mA4)
        {
            return _indicator.TRO_PMSM_Trend(input, mA1, mA2, mA3, mA4);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_PMSM_Trend TRO_PMSM_Trend(int mA1, int mA2, int mA3, int mA4)
        {
            return _indicator.TRO_PMSM_Trend(Input, mA1, mA2, mA3, mA4);
        }

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_PMSM_Trend TRO_PMSM_Trend(Data.IDataSeries input, int mA1, int mA2, int mA3, int mA4)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_PMSM_Trend(input, mA1, mA2, mA3, mA4);
        }

    }
}
#endregion
