/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO SMAA GridironTrade
/// Converted to NinjaTrader by cvax
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Interprets the TRO_SM_BattleLines in a radarscreen.")]
    [Gui.Design.DisplayName("TRO's Slot Machine AutoAvery Gridiron Trade")]
    public class TRO_SMAA_GridironTrade : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int			previousbar				= 0;		//Default period for fast swing-trade EMA
		private int			currentbar				= 0;		//Default period for slow swing-trade EMA
		private double		bottomPP				= 0;
		private double		topPP					= 0;
		private double		ballon					= 0;
		private string		ballfield;
		
        // User defined variables (add any user defined variables below)
		private	string						errorDisabled		= "GridironTrade disabled since either you are disconnected or current time is outside session time";
		private	string						errorIntraday		= "GridironTrade only works on intraday time based intervals";
		private System.Windows.Forms.Timer	timer;
		
		//Plot override
		private StringFormat			stringFormat		= new StringFormat();
		private StringFormat			stringFormatCenter	= new StringFormat();
		private	SolidBrush				textBrush			= new SolidBrush(Color.Black);
		private	SolidBrush				whiteBrush			= new SolidBrush(Color.White);
		private	System.Drawing.Font		textFont			= new Font("Arial", 10);
		private	System.Drawing.Font		textFontBold		= new Font("Arial", 10, FontStyle.Bold);
		
		private StopWatch				s					= new StopWatch();
		
        #endregion
        protected override void Initialize()
        {
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 2)	return;
			
			//Trade
			if(Open[1] > Close[1]) previousbar = -1;
			else if (Open[1] < Close[1]) previousbar = 1;
			else previousbar = 0;
			
			//Possession and TOP
			if(Open[0] > Close[0])
			{
				if(s.running==true)
				{
					if(currentbar != -1)
					{
						s.Stop();
						s.Start();
					}
				}
				else
					s.Start();
				currentbar = -1;
			}
			else if (Open[0] < Close[0])
			{
				if(s.running==true)
				{
					if(currentbar != 1)
					{
						s.Stop();
						s.Start();
					}
				}
				else
					s.Start();
				currentbar = 1;
			}
			else
			{
				currentbar = 0;
			}
			
			//Ball On
			#region R1-R5
			if (Close[0] > TRO_SM_BattleLines(Close).PP[0])
			{
				if (Close[0] > TRO_SM_BattleLines(Close).R1[0])
				{
					if(Close[0] > TRO_SM_BattleLines(Close).R2[0])
					{
						if(Close[0] > TRO_SM_BattleLines(Close).R3[0])
						{
							if(Close[0] > TRO_SM_BattleLines(Close).R4[0])
							{
								if(Close[0] > TRO_SM_BattleLines(Close).R5[0])
								{
									bottomPP = TRO_SM_BattleLines(Close).R5[0];
									topPP = 1000;
								}
								else
								{
									bottomPP = TRO_SM_BattleLines(Close).R4[0];
									topPP = TRO_SM_BattleLines(Close).R5[0];
								}
							}
							else
							{
								bottomPP = TRO_SM_BattleLines(Close).R3[0];
								topPP = TRO_SM_BattleLines(Close).R4[0];
							}
						}
						else
						{
							bottomPP = TRO_SM_BattleLines(Close).R2[0];
							topPP = TRO_SM_BattleLines(Close).R3[0];
						}
					}
					else
					{
						bottomPP = TRO_SM_BattleLines(Close).R1[0];
						topPP = TRO_SM_BattleLines(Close).R2[0];
					}
				}
				else
				{
					bottomPP = TRO_SM_BattleLines(Close).PP[0];
					topPP = TRO_SM_BattleLines(Close).R1[0];
				}
			}
			#endregion
			
			#region S1-S5
			if (Close[0] < TRO_SM_BattleLines(Close).PP[0])
			{
				if (Close[0] < TRO_SM_BattleLines(Close).S1[0])
				{
					if(Close[0] < TRO_SM_BattleLines(Close).S2[0])
					{
						if(Close[0] < TRO_SM_BattleLines(Close).S3[0])
						{
							if(Close[0] < TRO_SM_BattleLines(Close).S4[0])
							{
								if(Close[0] < TRO_SM_BattleLines(Close).S5[0])
								{
									topPP = TRO_SM_BattleLines(Close).S5[0];
									bottomPP = 0;
								}
								else
								{
									topPP = TRO_SM_BattleLines(Close).S4[0];
									bottomPP = TRO_SM_BattleLines(Close).S5[0];
								}
							}
							else
							{
								topPP = TRO_SM_BattleLines(Close).S3[0];
								bottomPP = TRO_SM_BattleLines(Close).S4[0];
							}
						}
						else
						{
							topPP = TRO_SM_BattleLines(Close).S2[0];
							bottomPP = TRO_SM_BattleLines(Close).S3[0];
						}
					}
					else
					{
						topPP = TRO_SM_BattleLines(Close).S1[0];
						bottomPP = TRO_SM_BattleLines(Close).S2[0];
					}
				}
				else
				{
					topPP = TRO_SM_BattleLines(Close).PP[0];
					bottomPP = TRO_SM_BattleLines(Close).S1[0];
				}
			}
			#endregion
			
			ballon = ((Close[0]-bottomPP)/(topPP-bottomPP))*100;
			if (ballon > 50)
			{
				ballon = 100-ballon;
				ballfield = "sellers";
			}
			else if (ballon < 50)
				ballfield = "buyers";
			else
				ballfield="midfield";
        }
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. Uncomment if indicators holds at least one plot.
			 base.Plot(graphics, bounds, min, max);

			// fill the complete paintable region
			// note: although the rectangle is greater and even would start plotting at the left upper corner of the chart, 
			// it's clipped to the paintable region
			//graphics.FillRectangle(new SolidBrush(Color.LightGray), new Rectangle (0, 0, 2000, 2000));

			// plot a green line from the upper left to the lower right corner
			// all painting needs to go by bounds X/Y offset
			//graphics.DrawLine(new Pen(Color.Green), bounds.X, bounds.Y, bounds.X + bounds.Width, bounds.Y + bounds.Height);

			// plot a green line from the lower left to the upper right corner and apply 
			// apply AnitAlias to make the line look nicer
			// don't forget to reset after plotting -> smoothing has some performance impact
			//SmoothingMode oldSmoothingMode = graphics.SmoothingMode;		// save current smoothing mode
			//graphics.SmoothingMode = SmoothingMode.AntiAlias;				// apply smoothing mode
			//graphics.DrawLine(new Pen(Color.Green), bounds.X, bounds.Y + bounds.Height, bounds.X + bounds.Width, bounds.Y);
			//graphics.SmoothingMode = oldSmoothingMode;						// restore smoothing mode

			graphics.FillRectangle(new SolidBrush(Color.White), new Rectangle (0, 0, 2000, 2000));
			
			// plot text in the upper left corner at position 10/10
			stringFormatCenter.Alignment = StringAlignment.Center;			// text is centered
			textBrush.Color			= Color.Black;							// set text color
			whiteBrush.Color			= Color.White;
			
			#region Column One
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawString("Trade", textFontBold, textBrush, bounds.X + 50, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(previousbar==1)
			{
				if(currentbar==-1)
				{
					graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("Trade", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Indigo), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("Wait", textFont, whiteBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
			}
			else if (previousbar==-1)
			{
				if(currentbar==1)
				{
					graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("Trade", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Indigo), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("Wait", textFont, whiteBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Indigo), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("Wait", textFont, whiteBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Two
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawString("Possession", textFontBold, textBrush, bounds.X + 130, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(currentbar==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("Buyers", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else if (currentbar==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("Sellers", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("Neutral", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Three
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawString("T.O.P.", textFontBold, textBrush, bounds.X + 210, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			string time = s.GetElapsedTimeSecs().Hours.ToString("00") + ":" + s.GetElapsedTimeSecs().Minutes.ToString("00") + ":" + s.GetElapsedTimeSecs().Seconds.ToString("00");
			graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
			graphics.DrawString(time, textFont, whiteBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			#endregion
			
			#region Column Four
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawString("Ball On", textFontBold, textBrush, bounds.X + 290, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(ballfield=="buyers")
			{
				if(ballon<=20)
				{
					graphics.FillRectangle(new SolidBrush(Color.DarkSeaGreen), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawString(Convert.ToString(double.Parse(ballon.ToString(".00"))), textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawString(Convert.ToString(double.Parse(ballon.ToString(".00"))), textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
				}
			}
			else if (ballfield=="sellers")
			{
				if(ballon<=20)
				{
					graphics.FillRectangle(new SolidBrush(Color.DarkRed), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawString(Convert.ToString(double.Parse(ballon.ToString(".00"))), textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
					graphics.DrawString(Convert.ToString(double.Parse(ballon.ToString(".00"))), textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
				}
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString(Convert.ToString(double.Parse(ballon.ToString(".00"))), textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
		}

        #region Properties
        #endregion
		
		private class StopWatch 
		{
			
			private DateTime startTime;
			private DateTime stopTime;
			public bool running = false;
			public TimeSpan interval;
		
			
			public void Start() 
			{
				this.startTime = DateTime.Now;
				this.running = true;
			}
		
		
			public void Stop() 
			{
				this.stopTime = DateTime.Now;
				this.running = false;
			}
		
		
			// elaspsed time in milliseconds
			public double GetElapsedTime() 
			{
				if (running) 
					interval = DateTime.Now - startTime;
				else
					interval = stopTime - startTime;
					
				return interval.TotalMilliseconds;
			}
			
			
			// elaspsed time in seconds
			public TimeSpan GetElapsedTimeSecs() 
			{
				if (running) 
					interval = DateTime.Now - startTime;
				else
					interval = stopTime - startTime;
					
				return interval;
			}
		}
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_SMAA_GridironTrade[] cacheTRO_SMAA_GridironTrade = null;

        private static TRO_SMAA_GridironTrade checkTRO_SMAA_GridironTrade = new TRO_SMAA_GridironTrade();

        /// <summary>
        /// Interprets the TRO_SM_BattleLines in a radarscreen.
        /// </summary>
        /// <returns></returns>
        public TRO_SMAA_GridironTrade TRO_SMAA_GridironTrade()
        {
            return TRO_SMAA_GridironTrade(Input);
        }

        /// <summary>
        /// Interprets the TRO_SM_BattleLines in a radarscreen.
        /// </summary>
        /// <returns></returns>
        public TRO_SMAA_GridironTrade TRO_SMAA_GridironTrade(Data.IDataSeries input)
        {

            if (cacheTRO_SMAA_GridironTrade != null)
                for (int idx = 0; idx < cacheTRO_SMAA_GridironTrade.Length; idx++)
                    if (cacheTRO_SMAA_GridironTrade[idx].EqualsInput(input))
                        return cacheTRO_SMAA_GridironTrade[idx];

            TRO_SMAA_GridironTrade indicator = new TRO_SMAA_GridironTrade();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            TRO_SMAA_GridironTrade[] tmp = new TRO_SMAA_GridironTrade[cacheTRO_SMAA_GridironTrade == null ? 1 : cacheTRO_SMAA_GridironTrade.Length + 1];
            if (cacheTRO_SMAA_GridironTrade != null)
                cacheTRO_SMAA_GridironTrade.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_SMAA_GridironTrade = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Interprets the TRO_SM_BattleLines in a radarscreen.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SMAA_GridironTrade TRO_SMAA_GridironTrade()
        {
            return _indicator.TRO_SMAA_GridironTrade(Input);
        }

        /// <summary>
        /// Interprets the TRO_SM_BattleLines in a radarscreen.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SMAA_GridironTrade TRO_SMAA_GridironTrade(Data.IDataSeries input)
        {
            return _indicator.TRO_SMAA_GridironTrade(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Interprets the TRO_SM_BattleLines in a radarscreen.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SMAA_GridironTrade TRO_SMAA_GridironTrade()
        {
            return _indicator.TRO_SMAA_GridironTrade(Input);
        }

        /// <summary>
        /// Interprets the TRO_SM_BattleLines in a radarscreen.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SMAA_GridironTrade TRO_SMAA_GridironTrade(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_SMAA_GridironTrade(input);
        }

    }
}
#endregion
