/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO SMAA High Low Break
/// Version: 1.01
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.01 - Fixed a major bug relating to value updates on Radar Screen during real-time.
/// 
/// Notes:  In Intraday timeframes the indicator will plot previous day's High-Low, otherwise it will show the Radar Screen.
/// 		Use the Statistics options to turn on/off the stat columns.
/// 
/// Usage:	Break Up = Current Day High - Previous Day High
/// 		Break Dn = Previous Day Low - Current Day Low
/// 		Range	 = Current Day High - Current Day Low
/// 		Range[1] = Previous Day High - Previous Day Low
/// 		Range and Range[1] will have green text if they are within Avg Range+-Std Dev. (68.1%) 
/// 			Orange text if Avg Range+2 Std Dev > Range > Avg Range+Std Dev or Avg Range-Std Dev > Range > Avg Range-2 Std Dev. (95.45%)
/// 			Red text if Avg Range+3 Std Dev > Range > Avg Range+2 Std Dev or Avg Range-2 Std Dev > Range > Avg Range-3 Std Dev. (99.73%)
/// 			Red background if Range > Avg Range+3 Std Dev or Avg Range-3 Std Dev > Range. 
/// 		(red bkgd=Range-3SD=red=Range-2SD=orange=Range-SD=green=Range=green=Range+SD=orange=Range+2SD=red=Range+3SD=red bkgd)
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Statistical trading with High-Low breakouts.")]
    [Gui.Design.DisplayName("TRO's Slot Machine AutoAvery High-Low Breakout")]
    public class TRO_SMAA_HLBreak : Indicator
    {
        #region Variables
        // Wizard generated variables
		private bool		stats			= true;
		
        // User defined variables (add any user defined variables below)
		private double		breakup			= 0.00;
		private double		breakdn			= 0.00;
		private double		prevrange		= 0;
		private double		range			= 0;
		private	double		avgrangecount	= 0;
		private double		avgrange		= 0;
		private double		sd				= 0;
		private double		priorhigh		= 0;
		private double		priorlow		= 0;
		private int			barcounter		= 0;
		private int			x				= 0;
		private int			rangecol		= 0;
		private int			range1col		= 0;
		
		//Plot override (RadarScreen)
		private StringFormat			stringFormat		= new StringFormat();
		private StringFormat			stringFormatCenter	= new StringFormat();
		private	SolidBrush				textBrush			= new SolidBrush(Color.Black);
		private	SolidBrush				whiteBrush			= new SolidBrush(Color.White);
		private	SolidBrush				greenBrush			= new SolidBrush(Color.LimeGreen);
		private	SolidBrush				redBrush			= new SolidBrush(Color.Red);
		private	SolidBrush				orangeBrush			= new SolidBrush(Color.Orange);
		private	System.Drawing.Font		textFont			= new Font("Arial", 10);
		private	System.Drawing.Font		textFontBold		= new Font("Arial", 10, FontStyle.Bold);
		
		//Plot override (Draw)
		private	SolidBrush[]			brushes				= { new SolidBrush(Color.Black), new SolidBrush(Color.Black)};
		private DateTime				currentDate			= Cbi.Globals.MinDate;

        #endregion
		
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Purple, 2), "PrevHigh"));
			Add(new Plot(new Pen(Color.DarkKhaki, 2), "PrevLow"));
			
			AutoScale			= false;
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= true;
			BarsRequired		= 1;
			stringFormat.Alignment = StringAlignment.Far;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 1)		return;
			
			currentDate = Time[0].Date;
			if (FirstTickOfBar)
			{
				priorhigh = PriorDayOHLC().PriorHigh[0];
				priorlow = PriorDayOHLC().PriorLow[0];
				prevhigh.Set(priorhigh);
				prevlow.Set(priorlow);
			}
			else
			{
				prevhigh.Set(priorhigh);
				prevlow.Set(priorlow);
			}
			
			if (Bars.Period.Id == PeriodType.Minute || Bars.Period.Id == PeriodType.Second || Bars.Period.Id == PeriodType.Tick || Bars.Period.Id == PeriodType.Volume)
				return;

			range = High[0] - Low[0];
			breakup = High[0] - High[1];
			breakdn = Low[1] - Low[0];
			if (breakdn < 0)
				breakdn = 0;
			if (breakup < 0)
				breakup = 0;
			if (FirstTickOfBar)
			{
				barcounter++;
				prevrange = High[1] - Low[1];

				avgrangecount = avgrangecount + range;
				avgrange = avgrangecount/(barcounter);
				//Standard Deviation calculations
				sd = 0;
				while ( x < barcounter)
				{
					sd = sd+(((High[x]-Low[x])-avgrange)*((High[x]-Low[x])-avgrange));
					x++;
				}
				x = 0;
				if (sd > 0)
					sd = System.Math.Sqrt(sd/(barcounter-1));
			}
        }
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars.Period.Id == PeriodType.Minute || Bars.Period.Id == PeriodType.Second || Bars.Period.Id == PeriodType.Tick || Bars.Period.Id == PeriodType.Volume)
			{
				#region Draw
				if (Bars == null || ChartControl == null)
					return;
	
				int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
				for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
				{
					SolidBrush		brush				= brushes[seriesCount];
					DateTime		lastDate			= Cbi.Globals.MinDate;
					DateTime		lastWeek			= Cbi.Globals.MinDate;
					DateTime		lastMonth			= Cbi.Globals.MinDate;
					int				firstX				= -1;
					int				lastX				= -1;
					int				lastY				= -1;
					int				xcounter			= 0;
					SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
					GraphicsPath	path				= new GraphicsPath();
					Gui.Chart.Plot	plot				= Plots[seriesCount];
					DataSeries	series				= (DataSeries) Values[seriesCount];
					
					if (brush.Color != plot.Pen.Color)	
						brush = new SolidBrush(plot.Pen.Color);
	
					for (int count = ChartControl.BarsPainted - 1; count >= ChartControl.BarsPainted - Math.Min(ChartControl.BarsPainted, 500); count--)
					{
						
						int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
						if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
							continue;
						else if (!series.IsValidPlot(idx))
							continue;
						
						if (lastDate == Cbi.Globals.MinDate)
							lastDate = Bars.Get(idx).Time.Date;
						else if (lastDate != Bars.Get(idx).Time.Date)
							continue;
						
	
						double		val = series.Get(idx);
						int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
											- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace);
						int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);
	
						if (xcounter == 0)
						{
							firstX	= x;
						}
						if (lastX >= 0)
						{
							if (y != lastY) // Problem here is, that last bar of old day has date of new day
								y = lastY;
							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
						lastX	= x;
						lastY	= y;
						
						xcounter++;
					}
	
					graphics.SmoothingMode = SmoothingMode.AntiAlias;
					graphics.DrawPath(plot.Pen, path);
					graphics.SmoothingMode = oldSmoothingMode;
					
					graphics.DrawString(plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);
				}
				#endregion
				return;
			}
			#region Radar Screen
			// Default plotting in base class. Uncomment if indicators holds at least one plot.
			base.Plot(graphics, bounds, min, max);

			graphics.FillRectangle(new SolidBrush(Color.White), new Rectangle (0, 0, 500, 88));
			
			// plot text in the upper left corner at position 10/10
			stringFormatCenter.Alignment = StringAlignment.Center;				// text is centered
			textBrush.Color				= Color.Black;							// set text color
			whiteBrush.Color			= Color.White;
			greenBrush.Color			= Color.LimeGreen;
			redBrush.Color				= Color.Red;
			orangeBrush.Color				= Color.Orange;
			
			#region Column One
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawString("Break Up", textFontBold, textBrush, bounds.X + 51, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
			if(breakup==0)
				graphics.DrawString("0.00", textFont, whiteBrush, bounds.X + 51, bounds.Y + 42, stringFormatCenter);
			else
				graphics.DrawString(Convert.ToString(double.Parse(breakup.ToString(".00"))), textFont, greenBrush, bounds.X + 51, bounds.Y + 42, stringFormatCenter);
			#endregion
			
			#region Column Two			
			//ROW 2
			graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 21);
			graphics.DrawRectangle(new Pen(Color.White), bounds.X + 90, bounds.Y + 40, 80, 21);
			if(breakdn==0)
				graphics.DrawString("0.00", textFont, whiteBrush, bounds.X + 131, bounds.Y + 42, stringFormatCenter);
			else
				graphics.DrawString(Convert.ToString(double.Parse(breakdn.ToString(".00"))), textFont, redBrush, bounds.X + 131, bounds.Y + 42, stringFormatCenter);
			
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawString("Break Dn", textFontBold, textBrush, bounds.X + 131, bounds.Y + 22, stringFormatCenter);
			#endregion
			
			#region Column Three			
			//ROW 2
			graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 21);
			graphics.DrawRectangle(new Pen(Color.White), bounds.X + 170, bounds.Y + 40, 80, 21);
			if(range<avgrange+sd && range>avgrange-sd)
				graphics.DrawString(Convert.ToString(double.Parse(range.ToString(".00"))), textFont, greenBrush, bounds.X + 211, bounds.Y + 42, stringFormatCenter);
			else if ((range>=avgrange+sd && range<avgrange+2*sd) || (range>avgrange-2*sd && range<=avgrange-sd))
				graphics.DrawString(Convert.ToString(double.Parse(range.ToString(".00"))), textFont, orangeBrush, bounds.X + 211, bounds.Y + 42, stringFormatCenter);
			else if ((range>=avgrange+2*sd && range<avgrange+3*sd) || (range>avgrange-3*sd && range<=avgrange-2*sd))
				graphics.DrawString(Convert.ToString(double.Parse(range.ToString(".00"))), textFont, redBrush, bounds.X + 211, bounds.Y + 42, stringFormatCenter);
			else if (range>=avgrange+3*sd || range<=avgrange-3*sd)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString(Convert.ToString(double.Parse(range.ToString(".00"))), textFont, textBrush, bounds.X + 211, bounds.Y + 42, stringFormatCenter);
				rangecol = 1;
			}
			
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawString("Range", textFontBold, textBrush, bounds.X + 211, bounds.Y + 22, stringFormatCenter);
			#endregion
			
			#region Column Four		
			//ROW 2
			if(rangecol == 0)
			{
				if(stats)
				{
					graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 21);
					graphics.DrawRectangle(new Pen(Color.White), bounds.X + 250, bounds.Y + 40, 80, 21);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 250, bounds.Y + 40, 81, 21);
					graphics.DrawRectangle(new Pen(Color.White), bounds.X + 250, bounds.Y + 40, 81, 21);
				}
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
			}
			if(prevrange<avgrange+sd && prevrange>avgrange-sd)
				graphics.DrawString(Convert.ToString(double.Parse(prevrange.ToString(".00"))), textFont, greenBrush, bounds.X + 291, bounds.Y + 42, stringFormatCenter);
			else if ((prevrange>=avgrange+sd && prevrange<avgrange+2*sd) || (prevrange>avgrange-2*sd && prevrange<=avgrange-sd))
				graphics.DrawString(Convert.ToString(double.Parse(prevrange.ToString(".00"))), textFont, orangeBrush, bounds.X + 291, bounds.Y + 42, stringFormatCenter);
			else if ((prevrange>=avgrange+2*sd && prevrange<avgrange+3*sd) || (prevrange>avgrange-3*sd && prevrange<=avgrange-2*sd))
				graphics.DrawString(Convert.ToString(double.Parse(prevrange.ToString(".00"))), textFont, redBrush, bounds.X + 291, bounds.Y + 42, stringFormatCenter);
			else if (prevrange>=avgrange+3*sd || prevrange<=avgrange-3*sd)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 250, bounds.Y + 40, 81, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 81, 20);
				graphics.DrawString(Convert.ToString(double.Parse(prevrange.ToString(".00"))), textFont, textBrush, bounds.X + 291, bounds.Y + 42, stringFormatCenter);
				range1col = 1;
			}
			
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawString("Range[1]", textFontBold, textBrush, bounds.X + 291, bounds.Y + 22, stringFormatCenter);
			#endregion
			
			if (stats)
			{
				#region Column Five				
				//ROW 2
				if(range1col == 0)
				{
					graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 330, bounds.Y + 40, 80, 21);
					graphics.DrawRectangle(new Pen(Color.White), bounds.X + 330, bounds.Y + 40, 80, 21);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 330, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 40, 80, 20);
				}
				graphics.DrawString(Convert.ToString(double.Parse(avgrange.ToString(".00"))), textFont, whiteBrush, bounds.X + 371, bounds.Y + 42, stringFormatCenter);
				
				//ROW 1
				graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 330, bounds.Y + 20, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 20, 80, 20);
				graphics.DrawString("Avg Range", textFontBold, textBrush, bounds.X + 371, bounds.Y + 22, stringFormatCenter);
				#endregion
	
				#region Column Six				
				//ROW 2
				graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 410, bounds.Y + 40, 81, 21);
				graphics.DrawRectangle(new Pen(Color.White), bounds.X + 410, bounds.Y + 40, 81, 21);
				graphics.DrawString(Convert.ToString(double.Parse(sd.ToString(".00"))), textFont, whiteBrush, bounds.X + 451, bounds.Y + 42, stringFormatCenter);
				
				//ROW 1
				graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 410, bounds.Y + 20, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 410, bounds.Y + 20, 80, 20);
				graphics.DrawString("Std Dev", textFontBold, textBrush, bounds.X + 451, bounds.Y + 22, stringFormatCenter);
				#endregion
			}
			#endregion
		}		
		
        #region Properties
		[Description("Show range statistics.")]
		[Category("High-Low Break Settings")]
		public bool Statistics 
		{
			get { return stats; }
			set { stats = value; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries prevhigh
		{
			get { return Values[0]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries prevlow
		{
			get { return Values[1]; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_SMAA_HLBreak[] cacheTRO_SMAA_HLBreak = null;

        private static TRO_SMAA_HLBreak checkTRO_SMAA_HLBreak = new TRO_SMAA_HLBreak();

        /// <summary>
        /// Statistical trading with High-Low breakouts.
        /// </summary>
        /// <returns></returns>
        public TRO_SMAA_HLBreak TRO_SMAA_HLBreak()
        {
            return TRO_SMAA_HLBreak(Input);
        }

        /// <summary>
        /// Statistical trading with High-Low breakouts.
        /// </summary>
        /// <returns></returns>
        public TRO_SMAA_HLBreak TRO_SMAA_HLBreak(Data.IDataSeries input)
        {

            if (cacheTRO_SMAA_HLBreak != null)
                for (int idx = 0; idx < cacheTRO_SMAA_HLBreak.Length; idx++)
                    if (cacheTRO_SMAA_HLBreak[idx].EqualsInput(input))
                        return cacheTRO_SMAA_HLBreak[idx];

            TRO_SMAA_HLBreak indicator = new TRO_SMAA_HLBreak();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            TRO_SMAA_HLBreak[] tmp = new TRO_SMAA_HLBreak[cacheTRO_SMAA_HLBreak == null ? 1 : cacheTRO_SMAA_HLBreak.Length + 1];
            if (cacheTRO_SMAA_HLBreak != null)
                cacheTRO_SMAA_HLBreak.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_SMAA_HLBreak = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Statistical trading with High-Low breakouts.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SMAA_HLBreak TRO_SMAA_HLBreak()
        {
            return _indicator.TRO_SMAA_HLBreak(Input);
        }

        /// <summary>
        /// Statistical trading with High-Low breakouts.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SMAA_HLBreak TRO_SMAA_HLBreak(Data.IDataSeries input)
        {
            return _indicator.TRO_SMAA_HLBreak(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Statistical trading with High-Low breakouts.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SMAA_HLBreak TRO_SMAA_HLBreak()
        {
            return _indicator.TRO_SMAA_HLBreak(Input);
        }

        /// <summary>
        /// Statistical trading with High-Low breakouts.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SMAA_HLBreak TRO_SMAA_HLBreak(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_SMAA_HLBreak(input);
        }

    }
}
#endregion
