/// TRO_SM_CandleColor2 8:21 AM 4/2/2007
/// 
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
///
/// DONATIONS AND GIFTS ACCEPTED 
///
/// P O BOX 43575, TUCSON, AZ 85733 
///
/// please include this and/or any other comment blocks and a 
/// description of any changes you make.
///
/// Copyright 2007, All Rights Reserved. Code may be modified for 
/// personal use only. No portion of this code can be sold or
/// otherwise redistributed without the express written consent of
/// the author, Avery T. Horton, Jr.
///
///
/// ATTN ESIGNAL: I DID NOT POST THIS CODE ON YOUR FORUM
///               I AM NOT RESPONSIBLE FOR ANY ESIGNAL FORUM RULE VIOLATIONS.
/// 
/// Indicator: TRO SM Candle Color 2
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.00 - Public Release

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
    [Description("Shows the current candle color in a slot machine display.")]
    [Gui.Design.DisplayName("TRO's Slot Machine Candle Color 2")]
    public class TRO_SM_CandleColor2 : Indicator
    {
        #region Variables
		private bool	drawhistory = true;
		private int		historylength = 7;
		
		private DataSeries candlecolor;
		private DataSeries candlediff;
		
		//Plot override
		private StringFormat			stringFormat		= new StringFormat();
		private StringFormat			stringFormatCenter	= new StringFormat();
		private	SolidBrush				textBrush			= new SolidBrush(Color.Black);
		private	System.Drawing.Font		textFont			= new Font("Arial", 10);
		private	System.Drawing.Font		textFontBold		= new Font("Arial", 10, FontStyle.Bold);
		private Color borderColor = Color.Transparent;
		private SolidBrush UpBrush = new SolidBrush(Color.Green);
		private SolidBrush DownBrush = new SolidBrush(Color.Red);
		private SolidBrush NeutralBrush = new SolidBrush(Color.Orange);
        #endregion

        protected override void Initialize()
        {
			candlecolor = new DataSeries(this);
			candlediff = new DataSeries(this);
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        protected override void OnBarUpdate()
        {
            if(Close[0]>Open[0])
				candlecolor.Set(1);
			else if(Close[0]==Open[0])
				candlecolor.Set(0);
			else
				candlecolor.Set(-1);
			candlediff.Set(Close[0]-Open[0]);
        }

        #region Properties
		[Description("Draw historical candles in a PMSM")]
		[Category("Candle Color")]
		[Gui.Design.DisplayName("Draw History")]
		public bool DrawHistory
		{
			get { return drawhistory; }
			set { drawhistory = value; }
		}
		
		[Description("# of historical candles in the PMSM")]
		[Category("Candle Color")]
		[Gui.Design.DisplayName("History Length")]
		public int HistoryLength
		{
			get { return historylength; }
			set { historylength = Math.Max(1,value); }
		}
        #endregion
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. Uncomment if indicators holds at least one plot.
			base.Plot(graphics, bounds, min, max);
			int bars = historylength-1;
			if(Bars.Count <= historylength-1)
			{
				graphics.DrawString("Not enough data. Need at least "+historylength+" bars.", textFontBold, textBrush, bounds.X + 50, bounds.Y + 22, stringFormatCenter);
				return;
			}
			graphics.FillRectangle(new SolidBrush(ChartControl.BackColor), new Rectangle (0, 0, 270 + 10*(bars-6), 88));
			
			// plot text in the upper left corner at position 10/10
			stringFormatCenter.Alignment = StringAlignment.Center;			// text is centered
			textBrush.Color			= Color.Black;							// set text color
			
			#region Column One
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawString("Candle", textFontBold, textBrush, bounds.X + 50, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			string difference = "";
			if(TickSize >= 1)
				difference = candlediff.Get(ChartControl.LastBarPainted).ToString();
			else if(TickSize >= 0.01 && TickSize < 1)
				difference = candlediff.Get(ChartControl.LastBarPainted).ToString("N2");
			else if (TickSize >= 0.001 && TickSize < 0.010)
				difference = candlediff.Get(ChartControl.LastBarPainted).ToString("N3");
			else if (TickSize >= 0.0001 && TickSize < 0.0010)
				difference = candlediff.Get(ChartControl.LastBarPainted).ToString("N4");
			else if (TickSize >= 0.00001 && TickSize < 0.00010)
				difference = candlediff.Get(ChartControl.LastBarPainted).ToString("N5");
			else if (TickSize < 0.000010)
				difference = candlediff.Get(ChartControl.LastBarPainted).ToString("N6");

			if(candlecolor.Get(ChartControl.LastBarPainted) == 1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString(difference, textFontBold, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			else if(candlecolor.Get(ChartControl.LastBarPainted) == -1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString(difference, textFontBold, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString(difference, textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Two
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawString("Interval", textFontBold, textBrush, bounds.X + 130, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			string interval = "";
			if(Bars.Period.Id == PeriodType.Minute)
				interval = Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Day)
				interval = "Daily";
			else if (Bars.Period.Id == PeriodType.Week)
				interval = "Weekly";
			else if (Bars.Period.Id == PeriodType.Month)
				interval = "Monthly";
			else if (Bars.Period.Id == PeriodType.Year)
				interval = "Yearly";
			else if (Bars.Period.Id == PeriodType.Tick)
				interval = "T"+Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Volume)
				interval = "V"+Bars.Period.Value.ToString();
			else if (Bars.Period.Id == PeriodType.Second)
				interval = "S"+Bars.Period.Value.ToString();
			if(candlecolor.Get(ChartControl.LastBarPainted) == 1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else if (candlecolor.Get(ChartControl.LastBarPainted) == -1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Historical Candles
			if(drawhistory)
			{
				if(bars<=6)
				{
					//ROW 1
					graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 170, bounds.Y + 20, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 20, 80, 20);
					graphics.DrawString("History", textFontBold, textBrush, bounds.X + 210, bounds.Y + 22, stringFormatCenter);

					//ROW 2
					graphics.FillRectangle(new SolidBrush(Color.Khaki), bounds.X + 170, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				}
				else
				{
					//ROW 1
					graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 170, bounds.Y + 20, 80 + 10*(bars-6), 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 20, 80 + 10*(bars-6), 20);
					graphics.DrawString("History", textFontBold, textBrush, bounds.X + 210 + 10*(bars-6)/2, bounds.Y + 22, stringFormatCenter);

					//ROW 2
					graphics.FillRectangle(new SolidBrush(Color.Khaki), bounds.X + 170, bounds.Y + 40, 80 + 10*(bars-6), 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80 + 10*(bars-6), 20);
				}
				
				byte bRed 	= (byte)~(base.ChartControl.BackColor.R);
				byte bGreen = (byte)~(base.ChartControl.BackColor.G);
				byte bBlue 	= (byte)~(base.ChartControl.BackColor.B);
				borderColor = Color.FromArgb(bRed,bGreen,bBlue);

				#region PMSM Dots
				while (bars <= (historylength-1) && bars>=0)
				{
					int alertVal = 0;
					int x = 0;
					int y = bounds.Y+50;
					alertVal = (int)candlecolor.Get(ChartControl.LastBarPainted-bars);
					x = bounds.X+180+10*(historylength-1-bars);
					Color dotColor = Color.Transparent;
					
					//Coloring: -1=red, 0=orange, 1=green
					if (1==alertVal) dotColor = UpBrush.Color;
					else if (-1==alertVal) dotColor = DownBrush.Color;
					else if (0==alertVal) dotColor = NeutralBrush.Color;
					
					if (dotColor!=Color.Transparent)
					{								     
						SmoothingMode smoothingMode = graphics.SmoothingMode;
						graphics.SmoothingMode = SmoothingMode.AntiAlias;
						
						Pen pen = new Pen(borderColor, 7);
						pen.DashStyle = DashStyle.Dot;
						pen.DashCap = DashCap.Round;
										
						graphics.DrawPie(pen,x-1,y-1,2,2,0,360);
										
						pen = new Pen(dotColor, 5);
						pen.DashStyle = DashStyle.Dot;
						pen.DashCap = DashCap.Round;
						
						graphics.DrawRectangle(pen,x-2,y-2,2,2);
	
						graphics.SmoothingMode = smoothingMode; 
					}
					bars--;
				}
				#endregion
			}
			#endregion
		}
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_SM_CandleColor2[] cacheTRO_SM_CandleColor2 = null;

        private static TRO_SM_CandleColor2 checkTRO_SM_CandleColor2 = new TRO_SM_CandleColor2();

        /// <summary>
        /// Shows the current candle color in a slot machine display.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_CandleColor2 TRO_SM_CandleColor2()
        {
            return TRO_SM_CandleColor2(Input);
        }

        /// <summary>
        /// Shows the current candle color in a slot machine display.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_CandleColor2 TRO_SM_CandleColor2(Data.IDataSeries input)
        {

            if (cacheTRO_SM_CandleColor2 != null)
                for (int idx = 0; idx < cacheTRO_SM_CandleColor2.Length; idx++)
                    if (cacheTRO_SM_CandleColor2[idx].EqualsInput(input))
                        return cacheTRO_SM_CandleColor2[idx];

            TRO_SM_CandleColor2 indicator = new TRO_SM_CandleColor2();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            TRO_SM_CandleColor2[] tmp = new TRO_SM_CandleColor2[cacheTRO_SM_CandleColor2 == null ? 1 : cacheTRO_SM_CandleColor2.Length + 1];
            if (cacheTRO_SM_CandleColor2 != null)
                cacheTRO_SM_CandleColor2.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_SM_CandleColor2 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Shows the current candle color in a slot machine display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_CandleColor2 TRO_SM_CandleColor2()
        {
            return _indicator.TRO_SM_CandleColor2(Input);
        }

        /// <summary>
        /// Shows the current candle color in a slot machine display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_CandleColor2 TRO_SM_CandleColor2(Data.IDataSeries input)
        {
            return _indicator.TRO_SM_CandleColor2(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Shows the current candle color in a slot machine display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_CandleColor2 TRO_SM_CandleColor2()
        {
            return _indicator.TRO_SM_CandleColor2(Input);
        }

        /// <summary>
        /// Shows the current candle color in a slot machine display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_CandleColor2 TRO_SM_CandleColor2(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_SM_CandleColor2(input);
        }

    }
}
#endregion
