/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO SM Compass
/// Version: 1.1.1
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.1.1 - Fixed a potential bug resulting in the indicator not plotting
/// v1.1.0 - Allowed indicator to update to the visible chart data rather than just the latest data
/// v1.0.0 - Public release
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Like a real compass, SM_Compass helps you determine which way a stock is heading.")]
    [Gui.Design.DisplayName("TRO's Slot Machine Compass")]
    public class TRO_SM_Compass : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int		ema1				= 13;		//Default period for fast swing-trade EMA
		private int		ema2				= 26;		//Default period for slow swing-trade EMA
		private int		ema3				= 5;		//Default period for scalp trade EMA
		private double ema1value			= 0.00;
		private double ema2value			= 0.00;
		private double ema3value			= 0.00;
		private double closevalue			= 0.00;
		
        // User defined variables (add any user defined variables below)
		private DataSeries		swingvalue;
		private DataSeries		swingcrossabove;
		private DataSeries		swingcrossbelow;
		private DataSeries		scalpvalue;
		private DataSeries		scalpcrossabove;
		private DataSeries		scalpcrossbelow;
		private DataSeries		rsivalue;
		
		
		//Plot override
		private StringFormat			stringFormat		= new StringFormat();
		private StringFormat			stringFormatCenter	= new StringFormat();
		private	SolidBrush				textBrush			= new SolidBrush(Color.Black);
		private	SolidBrush				rsiBrush			= new SolidBrush(Color.White);
		private	System.Drawing.Font		textFont			= new Font("Arial", 10);
		private	System.Drawing.Font		textFontBold		= new Font("Arial", 10, FontStyle.Bold);
		
        #endregion
        protected override void Initialize()
        {
			swingvalue = new DataSeries(this);
			swingcrossabove = new DataSeries(this);
			swingcrossbelow = new DataSeries(this);
			scalpvalue = new DataSeries(this);
			scalpcrossabove = new DataSeries(this);
			scalpcrossbelow = new DataSeries(this);
			rsivalue = new DataSeries(this);
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= true;
			PaintPriceMarkers	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < Math.Min(Math.Min(ema1,ema2),ema3)+1)	return;
			
			ema1value = double.Parse(EMA(ema1)[0].ToString(".00"));
			ema2value = double.Parse(EMA(ema2)[0].ToString(".00"));
			ema3value = double.Parse(EMA(ema3)[0].ToString(".00"));
			closevalue = double.Parse(Close[0].ToString(".00"));
			rsivalue.Set(double.Parse(RSI(Close,2,1)[0].ToString(".00")));
			
			//Swing-Trade EMA13xEMA26
			if (CrossAbove(EMA(ema1),EMA(ema2), 1))
			{
				swingcrossabove.Set(1);
				swingcrossbelow.Set(0);
				swingvalue.Set(1);
			}
			else if (CrossBelow(EMA(ema1),EMA(ema2), 1))
			{
				swingcrossabove.Set(0);
				swingcrossbelow.Set(1);
				swingvalue.Set(-1);
			}
			else if (ema1value > ema2value)
			{
				swingcrossabove.Set(0);
				swingcrossbelow.Set(0);
				swingvalue.Set(1);
			}
			else if (ema1value < ema2value)
			{
				swingcrossabove.Set(0);
				swingcrossbelow.Set(0);
				swingvalue.Set(-1);
			}
			else
			{
				swingcrossabove.Set(0);
				swingcrossbelow.Set(0);
				swingvalue.Set(0);
			}
			
			//Scalp-Trade ClosexEMA5
			if (CrossAbove(Close, EMA(ema3), 1))
			{
				scalpcrossabove.Set(1);
				scalpcrossbelow.Set(0);
				scalpvalue.Set(1);
			}
			else if (CrossBelow(Close,EMA(ema3), 1))
			{
				scalpcrossabove.Set(0);
				scalpcrossbelow.Set(1);
				scalpvalue.Set(-1);
			}
			else if (closevalue > ema3value)
			{
				scalpcrossabove.Set(0);
				scalpcrossbelow.Set(0);
				scalpvalue.Set(1);
			}
			else if (closevalue < ema3value)
			{
				scalpcrossabove.Set(0);
				scalpcrossbelow.Set(0);
				scalpvalue.Set(-1);
			}
			else
			{
				scalpcrossabove.Set(0);
				scalpcrossbelow.Set(0);
				scalpvalue.Set(0);
			}
        }
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. Uncomment if indicators holds at least one plot.
			 base.Plot(graphics, bounds, min, max);

			graphics.FillRectangle(new SolidBrush(Color.White), new Rectangle (0, 0, 350, 88));
			
			// plot text in the upper left corner at position 10/10
			stringFormatCenter.Alignment = StringAlignment.Center;			// text is centered
			textBrush.Color			= Color.Black;							// set text color
			rsiBrush.Color			= Color.White;
			
			#region Column One
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawString("Swing", textFontBold, textBrush, bounds.X + 50, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(swingvalue.Get(ChartControl.LastBarPainted)==1)
			{
				if(swingcrossabove.Get(ChartControl.LastBarPainted)==1)
				{
					graphics.FillRectangle(new SolidBrush(Color.DarkSeaGreen), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("+13x26", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("+", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
			}
			else if (swingvalue.Get(ChartControl.LastBarPainted)==-1)
			{
				if(swingcrossbelow.Get(ChartControl.LastBarPainted)==1)
				{
					graphics.FillRectangle(new SolidBrush(Color.DarkRed), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("-13x26", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
					graphics.DrawString("-", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
				}
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("=", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Two
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawString("Scalp", textFontBold, textBrush, bounds.X + 130, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(scalpvalue.Get(ChartControl.LastBarPainted)==1)
			{
				if(scalpcrossabove.Get(ChartControl.LastBarPainted)==1)
				{
					graphics.FillRectangle(new SolidBrush(Color.DarkSeaGreen), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawString("+Cx5", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawString("+", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
				}
			}
			else if (scalpvalue.Get(ChartControl.LastBarPainted)==-1)
			{
				if(scalpcrossbelow.Get(ChartControl.LastBarPainted)==1)
				{
					graphics.FillRectangle(new SolidBrush(Color.DarkRed), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawString("-Cx5", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
				}
				else
				{
					graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
					graphics.DrawString("-", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
				}
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("=", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Three
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawString("RSI2", textFontBold, textBrush, bounds.X + 210, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if (rsivalue.Get(ChartControl.LastBarPainted) >= 99)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString(rsivalue.Get(ChartControl.LastBarPainted).ToString(), textFont, textBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			else if (rsivalue.Get(ChartControl.LastBarPainted) <= 1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString(rsivalue.Get(ChartControl.LastBarPainted).ToString(), textFont, textBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString(rsivalue.Get(ChartControl.LastBarPainted).ToString(), textFont, rsiBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Four
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawString("Interval", textFontBold, textBrush, bounds.X + 290, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			string interval = "";
			if(Bars.Period.Id == PeriodType.Minute)
				interval = Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Day)
				interval = "Daily";
			else if (Bars.Period.Id == PeriodType.Week)
				interval = "Weekly";
			else if (Bars.Period.Id == PeriodType.Month)
				interval = "Monthly";
			else if (Bars.Period.Id == PeriodType.Year)
				interval = "Yearly";
			else if (Bars.Period.Id == PeriodType.Tick)
				interval = "T"+Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Volume)
				interval = "V"+Bars.Period.Value.ToString();
			else if (Bars.Period.Id == PeriodType.Second)
				interval = "S"+Bars.Period.Value.ToString();

			if(swingvalue.Get(ChartControl.LastBarPainted)==1 && scalpvalue.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			else if (swingvalue.Get(ChartControl.LastBarPainted)==-1 && scalpvalue.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
		}

        #region Properties
		[Description("Period for fast swing-trade EMA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int EMA1
		{
			get { return ema1; }
			set { ema1 = Math.Max(1, value); }
		}

		[Description("Period for slow swing-trade EMA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int EMA2
		{
			get { return ema2; }
			set { ema2 = Math.Max(1, value); }
		}
		
		[Description("Period for scalp-trade EMA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int EMA3
		{
			get { return ema3; }
			set { ema3 = Math.Max(1, value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_SM_Compass[] cacheTRO_SM_Compass = null;

        private static TRO_SM_Compass checkTRO_SM_Compass = new TRO_SM_Compass();

        /// <summary>
        /// Like a real compass, SM_Compass helps you determine which way a stock is heading.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_Compass TRO_SM_Compass(int eMA1, int eMA2, int eMA3)
        {
            return TRO_SM_Compass(Input, eMA1, eMA2, eMA3);
        }

        /// <summary>
        /// Like a real compass, SM_Compass helps you determine which way a stock is heading.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_Compass TRO_SM_Compass(Data.IDataSeries input, int eMA1, int eMA2, int eMA3)
        {
            checkTRO_SM_Compass.EMA1 = eMA1;
            eMA1 = checkTRO_SM_Compass.EMA1;
            checkTRO_SM_Compass.EMA2 = eMA2;
            eMA2 = checkTRO_SM_Compass.EMA2;
            checkTRO_SM_Compass.EMA3 = eMA3;
            eMA3 = checkTRO_SM_Compass.EMA3;

            if (cacheTRO_SM_Compass != null)
                for (int idx = 0; idx < cacheTRO_SM_Compass.Length; idx++)
                    if (cacheTRO_SM_Compass[idx].EMA1 == eMA1 && cacheTRO_SM_Compass[idx].EMA2 == eMA2 && cacheTRO_SM_Compass[idx].EMA3 == eMA3 && cacheTRO_SM_Compass[idx].EqualsInput(input))
                        return cacheTRO_SM_Compass[idx];

            TRO_SM_Compass indicator = new TRO_SM_Compass();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.EMA1 = eMA1;
            indicator.EMA2 = eMA2;
            indicator.EMA3 = eMA3;

            TRO_SM_Compass[] tmp = new TRO_SM_Compass[cacheTRO_SM_Compass == null ? 1 : cacheTRO_SM_Compass.Length + 1];
            if (cacheTRO_SM_Compass != null)
                cacheTRO_SM_Compass.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_SM_Compass = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Like a real compass, SM_Compass helps you determine which way a stock is heading.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_Compass TRO_SM_Compass(int eMA1, int eMA2, int eMA3)
        {
            return _indicator.TRO_SM_Compass(Input, eMA1, eMA2, eMA3);
        }

        /// <summary>
        /// Like a real compass, SM_Compass helps you determine which way a stock is heading.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_Compass TRO_SM_Compass(Data.IDataSeries input, int eMA1, int eMA2, int eMA3)
        {
            return _indicator.TRO_SM_Compass(input, eMA1, eMA2, eMA3);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Like a real compass, SM_Compass helps you determine which way a stock is heading.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_Compass TRO_SM_Compass(int eMA1, int eMA2, int eMA3)
        {
            return _indicator.TRO_SM_Compass(Input, eMA1, eMA2, eMA3);
        }

        /// <summary>
        /// Like a real compass, SM_Compass helps you determine which way a stock is heading.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_Compass TRO_SM_Compass(Data.IDataSeries input, int eMA1, int eMA2, int eMA3)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_SM_Compass(input, eMA1, eMA2, eMA3);
        }

    }
}
#endregion
