/****
TRO_SM_CONTROL_PMSM6 8:04 PM 4/1/2007 WHO'S IN CONTROL INDICATOR POOR MAN'S SLOT MACHINE VERSION
Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
DONATIONS AND GIFTS ACCEPTED 
P O BOX 43575, TUCSON, AZ 85733 

please include this and/or any other comment blocks and a 
description of any changes you make.
****/

/****
How I use TRO_SM_CONTROL_PMSM6: 
The Major/Minor daily is the row that dictates which way to trade. 
If buyer/buyers then I look to buy. 
If seller/sellers then I look to short. 
If Major and Minor disagree... I WAIT. NO TRADING. 
Then, I look at the 60 minute row. Same rules as daily. 
If I am a buyer and the 1/5/15 rows all say seller, I know the buyers will be pushing the sellers back soon. 
When I see the price stop moving at DYNAMIC SUPPORT, then I buy. 
When I see the price stop at dynamic resistance and the sellers take control on the 1 or 5, I sell. 
This is so simple it's not funny. 
All you have to do is follow the rules. 

Focus on:
1) who's in control ?
2) is price at/near dynamic s/r 
****/

/// Indicator: TRO SM Control PMSM6
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.00 - Public Release

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
    [Description("Plots the trend of moving averages in a linear display.")]
    [Gui.Design.DisplayName("TRO's Control PMSM6")]
    public class TRO_SM_Control_PMSM6 : Indicator
    {
        #region Variables
		private bool	drawhistory = true;
		private int		historylength = 7;
		private DataSeries candlecolor;
		private DataSeries Maj;
		private DataSeries MajPIR;
		private DataSeries Min;
		private DataSeries MinPIR;
		
		private double currentOpen;
		private double currentHigh;
		private double currentLow;
		private double previousHigh;
		private double previousLow;
		private double currentMid;
		private double previousMid;
		private double DiffMin;
		private double DiffMaj;
		
		//Plot override
		private StringFormat			stringFormat		= new StringFormat();
		private StringFormat			stringFormatCenter	= new StringFormat();
		private	SolidBrush				textBrush			= new SolidBrush(Color.Black);
		private	System.Drawing.Font		textFont			= new Font("Arial", 10);
		private	System.Drawing.Font		textFontBold		= new Font("Arial", 10, FontStyle.Bold);
		private Color borderColor = Color.Transparent;
		private SolidBrush UpBrush = new SolidBrush(Color.Green);
		private SolidBrush DownBrush = new SolidBrush(Color.Red);
		private SolidBrush NeutralBrush = new SolidBrush(Color.Orange);
        #endregion

        protected override void Initialize()
        {
			candlecolor = new DataSeries(this);
			Maj = new DataSeries(this);
			MajPIR = new DataSeries(this);
			Min = new DataSeries(this);
			MinPIR = new DataSeries(this);

            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= false;
			
        }

        protected override void OnBarUpdate()
        {
			if ( CurrentBar < 2)	return;
			
			currentOpen = Open[0];
			currentHigh = High[0];
			currentLow = Low[0];
			previousHigh = High[1];
			previousLow = Low[1];
			
			currentMid = (currentHigh + currentLow) * .5;
			previousMid = (previousHigh + previousLow) * .5;
			
			DiffMin = (Close[0] - currentMid);
			DiffMaj = (Close[0] - previousMid);
			
			MajPIR.Set(PctInRange ( Close[0], previousHigh, previousLow ));
			MinPIR.Set(PctInRange ( Close[0], currentHigh, currentLow ));
			
			if( DiffMaj > 0 )
			{
				Maj.Set(1);
			}
			else if (DiffMaj < 0)
			{
				Maj.Set(-1);
			}
			else
			{
				Maj.Set(0);
			}
			
			if( DiffMin > 0 )
			{
				Min.Set(1);
			}
			else if (DiffMin < 0)
			{
				Min.Set(-1);
			}
			else
			{
				Min.Set(0);
			}
			
			if(Maj[0]==1 && Min[0]==1)
			{
				candlecolor.Set(1);
			}
			else if(Maj[0]==-1 && Min[0]==-1)
			{
				candlecolor.Set(-1);
			}
			else
			{
				candlecolor.Set(0);
			}
        }

		#region Plot Override
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			base.Plot(graphics, bounds, min, max);
			stringFormatCenter.Alignment = StringAlignment.Center;			// text is centered
			textBrush.Color			= Color.Black;							// set text color
			
			int bars = historylength-1;
			if(Bars.Count <= historylength-1)
			{
				graphics.DrawString("Not enough data. Need at least "+historylength+" bars.", textFontBold, textBrush, bounds.X + 50, bounds.Y + 22, stringFormatCenter);
				return;
			}
			graphics.FillRectangle(new SolidBrush(ChartControl.BackColor), new Rectangle (0, 0, 270 + 10*(bars-6), 88));
			
			#region Column One
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawString("Major", textFontBold, textBrush, bounds.X + 50, bounds.Y + 20, stringFormatCenter);
			
			//ROW 2
			if((int)Maj.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("Buyers "+MajPIR.Get(ChartControl.LastBarPainted)+"%", textFont, textBrush, bounds.X + 50, bounds.Y + 40, stringFormatCenter);
			}
			else if ((int)Maj.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("Sellers "+MajPIR.Get(ChartControl.LastBarPainted)+"%", textFont, textBrush, bounds.X + 50, bounds.Y + 40, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("Even "+MajPIR.Get(ChartControl.LastBarPainted)+"%", textFont, textBrush, bounds.X + 50, bounds.Y + 40, stringFormatCenter);
			}
			#endregion
			
			#region Column Two
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawString("Minor", textFontBold, textBrush, bounds.X + 130, bounds.Y + 20, stringFormatCenter);
			
			//ROW 2
			if((int)Min.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("Buyers "+MinPIR.Get(ChartControl.LastBarPainted)+"%", textFont, textBrush, bounds.X + 130, bounds.Y + 40, stringFormatCenter);
			}
			else if ((int)Min.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("Sellers "+MinPIR.Get(ChartControl.LastBarPainted)+"%", textFont, textBrush, bounds.X + 130, bounds.Y + 40, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("Even "+MinPIR.Get(ChartControl.LastBarPainted)+"%", textFont, textBrush, bounds.X + 130, bounds.Y + 40, stringFormatCenter);
			}
			#endregion
			
			#region Column Three
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawString("Interval", textFontBold, textBrush, bounds.X + 210, bounds.Y + 20, stringFormatCenter);
			
			//ROW 2
			string interval = "";
			if(Bars.Period.Id == PeriodType.Minute)
				interval = Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Day)
				interval = "Daily";
			else if (Bars.Period.Id == PeriodType.Week)
				interval = "Weekly";
			else if (Bars.Period.Id == PeriodType.Month)
				interval = "Monthly";
			else if (Bars.Period.Id == PeriodType.Year)
				interval = "Yearly";
			else if (Bars.Period.Id == PeriodType.Tick)
				interval = "T"+Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Volume)
				interval = "V"+Bars.Period.Value.ToString();
			else if (Bars.Period.Id == PeriodType.Second)
				interval = "S"+Bars.Period.Value.ToString();
			if((int)Maj.Get(ChartControl.LastBarPainted)==1 && (int)Min.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 210, bounds.Y + 40, stringFormatCenter);
			}
			else if((int)Maj.Get(ChartControl.LastBarPainted)==-1 && (int)Min.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 210, bounds.Y + 40, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 210, bounds.Y + 40, stringFormatCenter);
			}
			#endregion
			
			#region Column Four
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawString("Trend", textFontBold, textBrush, bounds.X + 290, bounds.Y + 20, stringFormatCenter);
			
			//ROW 2
			int trendcount = 0;
			while((int)candlecolor.Get(ChartControl.LastBarPainted-trendcount)==(int)candlecolor.Get(ChartControl.LastBarPainted))
			{
				trendcount++;
			}
			if((int)candlecolor.Get(ChartControl.LastBarPainted)>0)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("+" + trendcount, textFont, textBrush, bounds.X + 290, bounds.Y + 40, stringFormatCenter);
			}
			else if ((int)candlecolor.Get(ChartControl.LastBarPainted)<0)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("-" + trendcount, textFont, textBrush, bounds.X + 290, bounds.Y + 40, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("=" + trendcount, textFont, textBrush, bounds.X + 290, bounds.Y + 40, stringFormatCenter);
			}
			#endregion
			
			#region Column Five
			if(drawhistory)
			{
				if(bars<=6)
				{
					//ROW 1
					graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 330, bounds.Y + 20, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 20, 80, 20);
					graphics.DrawString("History", textFontBold, textBrush, bounds.X + 370, bounds.Y + 20, stringFormatCenter);
					
					//ROW 2
					graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 330, bounds.Y + 40, 80, 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 40, 80, 20);
				}
				else
				{
					//ROW 1
					graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 330, bounds.Y + 20, 80 + 10*(bars-6), 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 20, 80 + 10*(bars-6), 20);
					graphics.DrawString("History", textFontBold, textBrush, bounds.X + 370 + 10*(bars-6)/2, bounds.Y + 20, stringFormatCenter);
					
					//ROW 2
					graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 330, bounds.Y + 40, 80 + 10*(bars-6), 20);
					graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 40, 80 + 10*(bars-6), 20);
				}
				
				byte bRed 	= (byte)~(base.ChartControl.BackColor.R);
				byte bGreen = (byte)~(base.ChartControl.BackColor.G);
				byte bBlue 	= (byte)~(base.ChartControl.BackColor.B);
				borderColor = Color.FromArgb(bRed,bGreen,bBlue);

				#region PMSM Dots
				while (bars <= (historylength-1) && bars>=0)
				{
					int alertVal = 0;
					int x = 0;
					int y = bounds.Y+50;
					alertVal = (int)candlecolor.Get(ChartControl.LastBarPainted-bars);
					x = bounds.X+340+10*(historylength-1-bars);
					Color dotColor = Color.Transparent;
					
					//Coloring: -1=red, 0=orange, 1=green
					if (1==alertVal) dotColor = UpBrush.Color;
					else if (-1==alertVal) dotColor = DownBrush.Color;
					else if (0==alertVal) dotColor = NeutralBrush.Color;
					
					if (dotColor!=Color.Transparent)
					{								     
						SmoothingMode smoothingMode = graphics.SmoothingMode;
						graphics.SmoothingMode = SmoothingMode.AntiAlias;
						
						Pen pen = new Pen(borderColor, 7);
						pen.DashStyle = DashStyle.Dot;
						pen.DashCap = DashCap.Round;
										
						graphics.DrawPie(pen,x-1,y-1,2,2,0,360);
										
						pen = new Pen(dotColor, 5);
						pen.DashStyle = DashStyle.Dot;
						pen.DashCap = DashCap.Round;
						
						graphics.DrawRectangle(pen,x-2,y-2,2,2);
	
						graphics.SmoothingMode = smoothingMode; 
					}
					bars--;
				}
				#endregion
			}
			#endregion
		}
		#endregion
		
		private double PctInRange ( double price, double high, double low )
		{
			double Pct = (price - low)/(high-low) * 100;
			if (Pct > 100)
				Pct = 100;
			else if (Pct < 0)
				Pct = 0;
			Pct = Math.Round(Pct);
			return Pct;
		}
		
        #region Properties
		[Description("Draw historical candles in a PMSM")]
		[Category("Candle Color")]
		[Gui.Design.DisplayName("Draw History")]
		public bool DrawHistory
		{
			get { return drawhistory; }
			set { drawhistory = value; }
		}
		
		[Description("# of historical candles in the PMSM")]
		[Category("Candle Color")]
		[Gui.Design.DisplayName("History Length")]
		public int HistoryLength
		{
			get { return historylength; }
			set { historylength = Math.Max(1,value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_SM_Control_PMSM6[] cacheTRO_SM_Control_PMSM6 = null;

        private static TRO_SM_Control_PMSM6 checkTRO_SM_Control_PMSM6 = new TRO_SM_Control_PMSM6();

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_Control_PMSM6 TRO_SM_Control_PMSM6()
        {
            return TRO_SM_Control_PMSM6(Input);
        }

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_Control_PMSM6 TRO_SM_Control_PMSM6(Data.IDataSeries input)
        {

            if (cacheTRO_SM_Control_PMSM6 != null)
                for (int idx = 0; idx < cacheTRO_SM_Control_PMSM6.Length; idx++)
                    if (cacheTRO_SM_Control_PMSM6[idx].EqualsInput(input))
                        return cacheTRO_SM_Control_PMSM6[idx];

            TRO_SM_Control_PMSM6 indicator = new TRO_SM_Control_PMSM6();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            TRO_SM_Control_PMSM6[] tmp = new TRO_SM_Control_PMSM6[cacheTRO_SM_Control_PMSM6 == null ? 1 : cacheTRO_SM_Control_PMSM6.Length + 1];
            if (cacheTRO_SM_Control_PMSM6 != null)
                cacheTRO_SM_Control_PMSM6.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_SM_Control_PMSM6 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_Control_PMSM6 TRO_SM_Control_PMSM6()
        {
            return _indicator.TRO_SM_Control_PMSM6(Input);
        }

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_Control_PMSM6 TRO_SM_Control_PMSM6(Data.IDataSeries input)
        {
            return _indicator.TRO_SM_Control_PMSM6(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_Control_PMSM6 TRO_SM_Control_PMSM6()
        {
            return _indicator.TRO_SM_Control_PMSM6(Input);
        }

        /// <summary>
        /// Plots the trend of moving averages in a linear display.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_Control_PMSM6 TRO_SM_Control_PMSM6(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_SM_Control_PMSM6(input);
        }

    }
}
#endregion
