/// <summary>
/// Programmer:  Avery T. Horton, Jr.  aka TheRumpledOne,  
/// DONATIONS AND GIFTS ACCEPTED 
/// P O BOX 43575, TUCSON, AZ 85733
///
/// Indicator: TRO SM Trend
/// Version: 1.2.1
/// Converted to NinjaTrader by cvax
/// 
/// Changelog:
/// v1.2.1 - Fixed missing Interval display for Volume and Tick time periods
/// v1.2.0 - Allowed indicator to update to the visible chart data rather than just the latest data
/// v1.1.0 - Added ability to choose moving average type
/// v1.0.0 - Public release
/// </summary>
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Plots the trend of simple moving averages in a table.")]
    [Gui.Design.DisplayName("TRO's Slot Machine Trend")]
    public class TRO_SM_Trend : Indicator
    {
        #region Variables
        // Parameter variables
		private int		ma1				= 5;				//Default period for first MA
		private int		ma2				= 10;				//Default period for second MA
		private int		ma3				= 20;				//Default period for third MA
		private int		ma4				= 40;				//Default period for fourth MA
		private TROAverageType	matype	= TROAverageType.EMA;	//Default moving average type
		
        // User defined variables (add any user defined variables below)
		private DataSeries		ma1value;
		private DataSeries		ma2value;
		private DataSeries		ma3value;
		private DataSeries		ma4value;
		private DataSeries		closevalue;		
		
		//Plot override
		private StringFormat			stringFormat		= new StringFormat();
		private StringFormat			stringFormatCenter	= new StringFormat();
		private	SolidBrush				textBrush			= new SolidBrush(Color.Black);
		private	System.Drawing.Font		textFont			= new Font("Arial", 10);
		private	System.Drawing.Font		textFontBold		= new Font("Arial", 10, FontStyle.Bold);
		
        #endregion
        protected override void Initialize()
        {
			ma1value = new DataSeries(this);
			ma2value = new DataSeries(this);
			ma3value = new DataSeries(this);
			ma4value = new DataSeries(this);
			closevalue = new DataSeries(this);
			
			CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= true;
			PaintPriceMarkers	= false;
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if ( CurrentBar < Math.Min(Math.Min(Math.Min(ma1,ma2),ma3),ma4)+1)	return;
			
			switch (matype)
			{
				case TROAverageType.EMA:
				{
					#region EMA
					//Close to 1st SMA
					if (Close[0] > EMA(ma1)[0])
					{
						closevalue.Set(1);
					}
					else if (Close[0] < EMA(ma1)[0])
					{
						closevalue.Set(-1);
					}
					else
					{
						closevalue.Set(0);
					}
					
					//1st SMA
					if (EMA(ma1)[0] > EMA(ma1)[1])
					{
						ma1value.Set(1);
					}
					else if (EMA(ma1)[0] < EMA(ma1)[1])
					{
						ma1value.Set(-1);
					}
					else
					{
						ma1value.Set(0);
					}
					
					//2nd SMA
					if (EMA(ma2)[0] > EMA(ma2)[1])
					{
						ma2value.Set(1);
					}
					else if (EMA(ma2)[0] < EMA(ma2)[1])
					{
						ma2value.Set(-1);
					}
					else
					{
						ma2value.Set(0);
					}
					
					//3rd SMA
					if (EMA(ma3)[0] > EMA(ma3)[1])
					{
						ma3value.Set(1);
					}
					else if (EMA(ma3)[0] < EMA(ma3)[1])
					{
						ma3value.Set(-1);
					}
					else
					{
						ma3value.Set(0);
					}
					
					//4th SMA
					if (EMA(ma4)[0] > EMA(ma4)[1])
					{
						ma4value.Set(1);
					}
					else if (EMA(ma4)[0] < EMA(ma4)[1])
					{
						ma4value.Set(-1);
					}
					else
					{
						ma4value.Set(0);
					}
					#endregion
					break;
				}
				case TROAverageType.HMA:
				{
					#region HMA
					//Close to 1st SMA
					if (Close[0] > HMA(ma1)[0])
					{
						closevalue.Set(1);
					}
					else if (Close[0] < HMA(ma1)[0])
					{
						closevalue.Set(-1);
					}
					else
					{
						closevalue.Set(0);
					}
					
					//1st SMA
					if (HMA(ma1)[0] > HMA(ma1)[1])
					{
						ma1value.Set(1);
					}
					else if (HMA(ma1)[0] < HMA(ma1)[1])
					{
						ma1value.Set(-1);
					}
					else
					{
						ma1value.Set(0);
					}
					
					//2nd SMA
					if (HMA(ma2)[0] > HMA(ma2)[1])
					{
						ma2value.Set(1);
					}
					else if (HMA(ma2)[0] < HMA(ma2)[1])
					{
						ma2value.Set(-1);
					}
					else
					{
						ma2value.Set(0);
					}
					
					//3rd SMA
					if (HMA(ma3)[0] > HMA(ma3)[1])
					{
						ma3value.Set(1);
					}
					else if (HMA(ma3)[0] < HMA(ma3)[1])
					{
						ma3value.Set(-1);
					}
					else
					{
						ma3value.Set(0);
					}
					
					//4th SMA
					if (HMA(ma4)[0] > HMA(ma4)[1])
					{
						ma4value.Set(1);
					}
					else if (HMA(ma4)[0] < HMA(ma4)[1])
					{
						ma4value.Set(-1);
					}
					else
					{
						ma4value.Set(0);
					}
					#endregion
					break;
				}
				case TROAverageType.SMA:
				{
					#region SMA
					//Close to 1st SMA
					if (Close[0] > SMA(ma1)[0])
					{
						closevalue.Set(1);
					}
					else if (Close[0] < SMA(ma1)[0])
					{
						closevalue.Set(-1);
					}
					else
					{
						closevalue.Set(0);
					}
					
					//1st SMA
					if (SMA(ma1)[0] > SMA(ma1)[1])
					{
						ma1value.Set(1);
					}
					else if (SMA(ma1)[0] < SMA(ma1)[1])
					{
						ma1value.Set(-1);
					}
					else
					{
						ma1value.Set(0);
					}
					
					//2nd SMA
					if (SMA(ma2)[0] > SMA(ma2)[1])
					{
						ma2value.Set(1);
					}
					else if (SMA(ma2)[0] < SMA(ma2)[1])
					{
						ma2value.Set(-1);
					}
					else
					{
						ma2value.Set(0);
					}
					
					//3rd SMA
					if (SMA(ma3)[0] > SMA(ma3)[1])
					{
						ma3value.Set(1);
					}
					else if (SMA(ma3)[0] < SMA(ma3)[1])
					{
						ma3value.Set(-1);
					}
					else
					{
						ma3value.Set(0);
					}
					
					//4th SMA
					if (SMA(ma4)[0] > SMA(ma4)[1])
					{
						ma4value.Set(1);
					}
					else if (SMA(ma4)[0] < SMA(ma4)[1])
					{
						ma4value.Set(-1);
					}
					else
					{
						ma4value.Set(0);
					}
					#endregion
					break;
				}
				case TROAverageType.WMA:
				{
					#region WMA
					//Close to 1st SMA
					if (Close[0] > WMA(ma1)[0])
					{
						closevalue.Set(1);
					}
					else if (Close[0] < WMA(ma1)[0])
					{
						closevalue.Set(-1);
					}
					else
					{
						closevalue.Set(0);
					}
					
					//1st SMA
					if (WMA(ma1)[0] > WMA(ma1)[1])
					{
						ma1value.Set(1);
					}
					else if (WMA(ma1)[0] < WMA(ma1)[1])
					{
						ma1value.Set(-1);
					}
					else
					{
						ma1value.Set(0);
					}
					
					//2nd SMA
					if (WMA(ma2)[0] > WMA(ma2)[1])
					{
						ma2value.Set(1);
					}
					else if (WMA(ma2)[0] < WMA(ma2)[1])
					{
						ma2value.Set(-1);
					}
					else
					{
						ma2value.Set(0);
					}
					
					//3rd SMA
					if (WMA(ma3)[0] > WMA(ma3)[1])
					{
						ma3value.Set(1);
					}
					else if (WMA(ma3)[0] < WMA(ma3)[1])
					{
						ma3value.Set(-1);
					}
					else
					{
						ma3value.Set(0);
					}
					
					//4th SMA
					if (WMA(ma4)[0] > WMA(ma4)[1])
					{
						ma4value.Set(1);
					}
					else if (WMA(ma4)[0] < WMA(ma4)[1])
					{
						ma4value.Set(-1);
					}
					else
					{
						ma4value.Set(0);
					}
					#endregion
					break;
				}
			}
        }
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. Uncomment if indicators holds at least one plot.
			 base.Plot(graphics, bounds, min, max);
			graphics.FillRectangle(new SolidBrush(Color.White), new Rectangle (0, 0, 420, 88));
			
			// plot text in the upper left corner at position 10/10
			stringFormatCenter.Alignment = StringAlignment.Center;			// text is centered
			textBrush.Color			= Color.Black;							// set text color
			
			#region Column One
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 20, 80, 20);
			graphics.DrawString(Convert.ToString(ma1), textFontBold, textBrush, bounds.X + 50, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(closevalue.Get(ChartControl.LastBarPainted)== 1 && ma1value.Get(ChartControl.LastBarPainted)==1 && ma2value.Get(ChartControl.LastBarPainted)==1 && ma3value.Get(ChartControl.LastBarPainted)==1 && ma4value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("", textFontBold, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			else if(closevalue.Get(ChartControl.LastBarPainted)== -1 && ma1value.Get(ChartControl.LastBarPainted)==-1 && ma2value.Get(ChartControl.LastBarPainted)==-1 && ma3value.Get(ChartControl.LastBarPainted)==-1 && ma4value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("D", textFontBold, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			else if(ma1value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("+", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			else if (ma1value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("-", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 10, bounds.Y + 40, 80, 20);
				graphics.DrawString("=", textFont, textBrush, bounds.X + 50, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Two
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 20, 80, 20);
			graphics.DrawString(Convert.ToString(ma2), textFontBold, textBrush, bounds.X + 130, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(closevalue.Get(ChartControl.LastBarPainted)== 1 && ma1value.Get(ChartControl.LastBarPainted)==1 && ma2value.Get(ChartControl.LastBarPainted)==1 && ma3value.Get(ChartControl.LastBarPainted)==1 && ma4value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.DarkSeaGreen), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("U", textFontBold, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else if(closevalue.Get(ChartControl.LastBarPainted)== -1 && ma1value.Get(ChartControl.LastBarPainted)==-1 && ma2value.Get(ChartControl.LastBarPainted)==-1 && ma3value.Get(ChartControl.LastBarPainted)==-1 && ma4value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("O", textFontBold, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else if(ma2value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("+", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else if (ma2value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("-", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 90, bounds.Y + 40, 80, 20);
				graphics.DrawString("=", textFont, textBrush, bounds.X + 130, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Three
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 20, 80, 20);
			graphics.DrawString(Convert.ToString(ma3), textFontBold, textBrush, bounds.X + 210, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(closevalue.Get(ChartControl.LastBarPainted)== 1 && ma1value.Get(ChartControl.LastBarPainted)==1 && ma2value.Get(ChartControl.LastBarPainted)==1 && ma3value.Get(ChartControl.LastBarPainted)==1 && ma4value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.DarkSeaGreen), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString("P", textFontBold, textBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			else if(closevalue.Get(ChartControl.LastBarPainted)== -1 && ma1value.Get(ChartControl.LastBarPainted)==-1 && ma2value.Get(ChartControl.LastBarPainted)==-1 && ma3value.Get(ChartControl.LastBarPainted)==-1 && ma4value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString("W", textFontBold, textBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			else if(ma3value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString("+", textFont, textBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			else if(ma3value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString("-", textFont, textBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 170, bounds.Y + 40, 80, 20);
				graphics.DrawString("=", textFont, textBrush, bounds.X + 210, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
			
			#region Column Four
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 20, 80, 20);
			graphics.DrawString(Convert.ToString(ma4), textFontBold, textBrush, bounds.X + 290, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			if(closevalue.Get(ChartControl.LastBarPainted)== 1 && ma1value.Get(ChartControl.LastBarPainted)==1 && ma2value.Get(ChartControl.LastBarPainted)==1 && ma3value.Get(ChartControl.LastBarPainted)==1 && ma4value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("", textFontBold, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			else if(closevalue.Get(ChartControl.LastBarPainted)== -1 && ma1value.Get(ChartControl.LastBarPainted)==-1 && ma2value.Get(ChartControl.LastBarPainted)==-1 && ma3value.Get(ChartControl.LastBarPainted)==-1 && ma4value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("N", textFontBold, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			else if(ma4value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("+", textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			else if (ma4value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("-", textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 250, bounds.Y + 40, 80, 20);
				graphics.DrawString("=", textFont, textBrush, bounds.X + 290, bounds.Y + 42, stringFormatCenter);
			}
			#endregion

			#region Column Five
			//ROW 1
			graphics.FillRectangle(new SolidBrush(Color.Gray), bounds.X + 330, bounds.Y + 20, 80, 20);
			graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 20, 80, 20);
			graphics.DrawString("Interval", textFontBold, textBrush, bounds.X + 370, bounds.Y + 22, stringFormatCenter);
			
			//ROW 2
			string interval = "";
			if(Bars.Period.Id == PeriodType.Minute)
				interval = Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Day)
				interval = "Daily";
			else if (Bars.Period.Id == PeriodType.Week)
				interval = "Weekly";
			else if (Bars.Period.Id == PeriodType.Month)
				interval = "Monthly";
			else if (Bars.Period.Id == PeriodType.Year)
				interval = "Yearly";
			else if (Bars.Period.Id == PeriodType.Tick)
				interval = "T"+Convert.ToString(Bars.Period.Value);
			else if (Bars.Period.Id == PeriodType.Volume)
				interval = "V"+Bars.Period.Value.ToString();
			else if (Bars.Period.Id == PeriodType.Second)
				interval = "S"+Bars.Period.Value.ToString();

			if(ma1value.Get(ChartControl.LastBarPainted)==1 && ma2value.Get(ChartControl.LastBarPainted)==1 && ma3value.Get(ChartControl.LastBarPainted)==1 && ma4value.Get(ChartControl.LastBarPainted)==1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Green), bounds.X + 330, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 370, bounds.Y + 42, stringFormatCenter);
			}
			else if (ma1value.Get(ChartControl.LastBarPainted)==-1 && ma2value.Get(ChartControl.LastBarPainted)==-1 && ma3value.Get(ChartControl.LastBarPainted)==-1 && ma4value.Get(ChartControl.LastBarPainted)==-1)
			{
				graphics.FillRectangle(new SolidBrush(Color.Red), bounds.X + 330, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 370, bounds.Y + 42, stringFormatCenter);
			}
			else
			{
				graphics.FillRectangle(new SolidBrush(Color.Gold), bounds.X + 330, bounds.Y + 40, 80, 20);
				graphics.DrawRectangle(new Pen(Color.Black), bounds.X + 330, bounds.Y + 40, 80, 20);
				graphics.DrawString(interval, textFont, textBrush, bounds.X + 370, bounds.Y + 42, stringFormatCenter);
			}
			#endregion
		}

        #region Properties
		[Description("Choose a Moving Average type.")]
		[Category("Parameters")]
		public TROAverageType MAType
		{
			get { return matype; }
			set { matype = value; }
		}
		
		[Description("Period for first MA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA1
		{
			get { return ma1; }
			set { ma1 = Math.Max(1, value); }
		}

		[Description("Period for second MA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA2
		{
			get { return ma2; }
			set { ma2 = Math.Max(1, value); }
		}
		
		[Description("Period for third MA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA3
		{
			get { return ma3; }
			set { ma3 = Math.Max(1, value); }
		}
		
		[Description("Period for fourth MA line. (Numbers of bars used for calculations)")]
		[Category("Parameters")]
		public int MA4
		{
			get { return ma4; }
			set { ma4 = Math.Max(1, value); }
		}
        #endregion
    }
}

public enum TROAverageType
{
	EMA,
	HMA,
	SMA,
	WMA,
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TRO_SM_Trend[] cacheTRO_SM_Trend = null;

        private static TRO_SM_Trend checkTRO_SM_Trend = new TRO_SM_Trend();

        /// <summary>
        /// Plots the trend of simple moving averages in a table.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_Trend TRO_SM_Trend(int mA1, int mA2, int mA3, int mA4)
        {
            return TRO_SM_Trend(Input, mA1, mA2, mA3, mA4);
        }

        /// <summary>
        /// Plots the trend of simple moving averages in a table.
        /// </summary>
        /// <returns></returns>
        public TRO_SM_Trend TRO_SM_Trend(Data.IDataSeries input, int mA1, int mA2, int mA3, int mA4)
        {
            checkTRO_SM_Trend.MA1 = mA1;
            mA1 = checkTRO_SM_Trend.MA1;
            checkTRO_SM_Trend.MA2 = mA2;
            mA2 = checkTRO_SM_Trend.MA2;
            checkTRO_SM_Trend.MA3 = mA3;
            mA3 = checkTRO_SM_Trend.MA3;
            checkTRO_SM_Trend.MA4 = mA4;
            mA4 = checkTRO_SM_Trend.MA4;

            if (cacheTRO_SM_Trend != null)
                for (int idx = 0; idx < cacheTRO_SM_Trend.Length; idx++)
                    if (cacheTRO_SM_Trend[idx].MA1 == mA1 && cacheTRO_SM_Trend[idx].MA2 == mA2 && cacheTRO_SM_Trend[idx].MA3 == mA3 && cacheTRO_SM_Trend[idx].MA4 == mA4 && cacheTRO_SM_Trend[idx].EqualsInput(input))
                        return cacheTRO_SM_Trend[idx];

            TRO_SM_Trend indicator = new TRO_SM_Trend();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MA1 = mA1;
            indicator.MA2 = mA2;
            indicator.MA3 = mA3;
            indicator.MA4 = mA4;

            TRO_SM_Trend[] tmp = new TRO_SM_Trend[cacheTRO_SM_Trend == null ? 1 : cacheTRO_SM_Trend.Length + 1];
            if (cacheTRO_SM_Trend != null)
                cacheTRO_SM_Trend.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTRO_SM_Trend = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots the trend of simple moving averages in a table.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_Trend TRO_SM_Trend(int mA1, int mA2, int mA3, int mA4)
        {
            return _indicator.TRO_SM_Trend(Input, mA1, mA2, mA3, mA4);
        }

        /// <summary>
        /// Plots the trend of simple moving averages in a table.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_Trend TRO_SM_Trend(Data.IDataSeries input, int mA1, int mA2, int mA3, int mA4)
        {
            return _indicator.TRO_SM_Trend(input, mA1, mA2, mA3, mA4);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots the trend of simple moving averages in a table.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TRO_SM_Trend TRO_SM_Trend(int mA1, int mA2, int mA3, int mA4)
        {
            return _indicator.TRO_SM_Trend(Input, mA1, mA2, mA3, mA4);
        }

        /// <summary>
        /// Plots the trend of simple moving averages in a table.
        /// </summary>
        /// <returns></returns>
        public Indicator.TRO_SM_Trend TRO_SM_Trend(Data.IDataSeries input, int mA1, int mA2, int mA3, int mA4)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TRO_SM_Trend(input, mA1, mA2, mA3, mA4);
        }

    }
}
#endregion
